<?php  // $Id: lib.php,v 1.16.8.2 2007/10/27 16:14:38 joseph_rezeau Exp $

require_once('locallib.php');

/// Library of functions and constants for module portfolio
/// (replace portfolio with the name of your module and delete this line)

define ('portfolioUNLIMITED', 0);
define ('portfolioONCE', 1);
define ('portfolioDAILY', 2);
define ('portfolioWEEKLY', 3);
define ('portfolioMONTHLY', 4);
$portfolio_TYPES = array (portfolioUNLIMITED => get_string('qtypeunlimited', 'portfolio'),
                              portfolioONCE => get_string('qtypeonce', 'portfolio'),
                              portfolioDAILY => get_string('qtypedaily', 'portfolio'),
                              portfolioWEEKLY => get_string('qtypeweekly', 'portfolio'),
                              portfolioMONTHLY => get_string('qtypemonthly', 'portfolio'));
$portfolio_RESPONDENTS = array ('fullname' => get_string('respondenttypefullname', 'portfolio'),
                                    'anonymous' => get_string('respondenttypeanonymous', 'portfolio'));
$portfolio_ELIGIBLES = array ('all' => get_string('respondenteligibleall', 'portfolio'),
                                  'students' => get_string('respondenteligiblestudents', 'portfolio'),
                                  'teachers' => get_string('respondenteligibleteachers', 'portfolio'));
$portfolio_REALMS = array ('private' => get_string('private', 'portfolio'),
                               'public' => get_string('public', 'portfolio'),
                               'template' => get_string('template', 'portfolio'));

$portfolio_STUDENTVIEWRESPONSES_NEVER = 0;
$portfolio_STUDENTVIEWRESPONSES_WHENANSWERED = 1;
$portfolio_STUDENTVIEWRESPONSES_WHENCLOSED = 2;
$portfolio_STUDENTVIEWRESPONSES_ALWAYS = 3;

$portfolio_RESPONSEVIEWERS =
    array ( $portfolio_STUDENTVIEWRESPONSES_NEVER => get_string('responseviewstudentsnever', 'portfolio'),
            $portfolio_STUDENTVIEWRESPONSES_WHENANSWERED => get_string('responseviewstudentswhenanswered', 'portfolio'),
            $portfolio_STUDENTVIEWRESPONSES_WHENCLOSED => get_string('responseviewstudentswhenclosed', 'portfolio'),
            $portfolio_STUDENTVIEWRESPONSES_ALWAYS => get_string('responseviewstudentsalways', 'portfolio'));

$portfolio_EDITING = 0;
$portfolio_ACTIVE1 = 1;
$portfolio_ENDED = 3;
$portfolio_ARCHIVED = 4;
$portfolio_TESTING = 8;
$portfolio_ACTIVE2 = 9;


function portfolio_add_instance($portfolio) {
/// Given an object containing all the necessary data, 
/// (defined by the form in mod.html) this function 
/// will create a new instance and return the id number 
/// of the new instance.


    // Check the realm and set it to the survey if it's set.
    if (!empty($portfolio->sid) && !empty($portfolio->realm)) {
        set_field('portfolio_survey', 'realm', $portfolio->realm, 'id', $portfolio->sid);
    }

    $portfolio->timemodified = time();

    # May have to add extra stuff in here #
    $portfolio->useopendate = (isset($portfolio->useopendate)) ? 1 : 0;
    $portfolio->useclosedate = (isset($portfolio->useclosedate)) ? 1 : 0;

    if ($portfolio->useopendate) {
        $portfolio->opendate =
            make_timestamp($portfolio->openyear, $portfolio->openmonth, $portfolio->openday,
                           $portfolio->openhour, $portfolio->openminute, 0);
    }
    else {
        $portfolio->opendate = 0;
    }

    if ($portfolio->useclosedate) {
        $portfolio->closedate =
            make_timestamp($portfolio->closeyear, $portfolio->closemonth, $portfolio->closeday,
                           $portfolio->closehour, $portfolio->closeminute, 0);
    }
    else {
        $portfolio->closedate = 0;
    }

    if ($portfolio->resume == '1') { //JR
        $portfolio->resume = 1;
    } else {
        $portfolio->resume = 0;
    }
    $portfolio->navigate = 1; // not used at all!

    return insert_record("portfolio", $portfolio);
}


function portfolio_update_instance($portfolio) {
/// Given an object containing all the necessary data, 
/// (defined by the form in mod.html) this function 
/// will update an existing instance with new data.

    // Check the realm and set it to the survey if its set.
    if (!empty($portfolio->sid) && !empty($portfolio->realm)) {
        set_field('portfolio_survey', 'realm', $portfolio->realm, 'id', $portfolio->sid);
    }

    $portfolio->timemodified = time();
    $portfolio->id = $portfolio->instance;

    # May have to add extra stuff in here #
    $portfolio->useopendate = (isset($portfolio->useopendate)) ? 1 : 0;
    $portfolio->useclosedate = (isset($portfolio->useclosedate)) ? 1 : 0;

    if ($portfolio->useopendate) {
        $portfolio->opendate =
            make_timestamp($portfolio->openyear, $portfolio->openmonth, $portfolio->openday,
                           $portfolio->openhour, $portfolio->openminute, 0);
    }
    else {
        $portfolio->opendate = 0;
    }

    if ($portfolio->useclosedate) {
        $portfolio->closedate =
            make_timestamp($portfolio->closeyear, $portfolio->closemonth, $portfolio->closeday,
                           $portfolio->closehour, $portfolio->closeminute, 0);
    }
    else {
        $portfolio->closedate = 0;
    }

    if ($portfolio->resume == '1') { //JR
        $portfolio->resume = 1;
    } else {
        $portfolio->resume = 0;
    }
    $portfolio->navigate = 1;

    return update_record("portfolio", $portfolio);
}


function portfolio_delete_instance($id) {
/// Given an ID of an instance of this module, 
/// this function will permanently delete the instance 
/// and any data that depends on it.  

    if (! $portfolio = get_record('portfolio', 'id', $id)) {
        return false;
    }

    $result = true;

    if (! delete_records('portfolio', 'id', $portfolio->id)) {
        $result = false;
    }

    if ($survey = get_record('portfolio_survey', 'id', $portfolio->sid)) {
    /// If this survey is owned by this course, delete all of the survey records and responses.
        if ($survey->owner == $portfolio->course) {
            $result = $result && portfolio_delete_survey($portfolio->sid, $portfolio->id);
        }
    }

    return $result;
}

function portfolio_user_outline($course, $user, $mod, $portfolio) {
/// Return a small object with summary information about what a 
/// user has done with a given particular instance of this module
/// Used for user activity reports.
/// $return->time = the time they did it
/// $return->info = a short text description
    $result = '';
    if ($responses = portfolio_get_user_responses($portfolio->sid, $user->id)) {
        $n = count($responses);
        if ($n == 1) {
            $result->info = $n.' '.get_string("response", "portfolio");            
        } else {
            $result->info = $n.' '.get_string("responses", "portfolio");            
        }
        $lastresponse = array_pop($responses);
        $result->time = $lastresponse->submitted;
    } else {
            $result->info = get_string("noresponses", "portfolio");                    
    }
        return $result;
}

/**
 * Get all the portfolio responses for a user
 */
function portfolio_get_user_responses($surveyid, $userid) {
    global $CFG;

    return get_records_sql ("SELECT *
        FROM {$CFG->prefix}portfolio_response
        WHERE survey_id = '$surveyid'
        AND username = '$userid'
        ORDER BY submitted ASC ");
}

function portfolio_user_complete($course, $user, $mod, $portfolio) {
/// Print a detailed representation of what a  user has done with 
/// a given particular instance of this module, for user activity reports.
    if ($responses = portfolio_get_user_responses($portfolio->sid, $user->id)) {
        foreach ($responses as $response) {
            echo get_string('submitted', 'portfolio').' '.userdate($response->submitted).'<br />';
        }
    } else {
       print_string('noresponses', 'portfolio');
    }

    return true;
}

function portfolio_print_recent_activity($course, $isteacher, $timestart) {
/// Given a course and a time, this module should find recent activity 
/// that has occurred in portfolio activities and print it out. 
/// Return true if there was output, or false is there was none.

    global $CFG;

    return false;  //  True if anything was printed, otherwise false 
}

function portfolio_cron () {
/// Function to be run periodically according to the moodle cron
/// This function searches for things that need to be done, such 
/// as sending out mail, toggling flags etc ... 

    global $CFG;

    return portfolio_cleanup();
}

function portfolio_grades($portfolioid) {
/// Must return an array of grades for a given instance of this module, 
/// indexed by user.  It also returns a maximum allowed grade.

    return NULL;
}

function portfolio_get_participants($portfolioid) {
//Must return an array of user records (all data) who are participants
//for a given instance of portfolio. Must include every user involved
//in the instance, independient of his role (student, teacher, admin...)
//See other modules as example.
    global $CFG;

    //Get students
    $users = get_records_sql('SELECT DISTINCT u.* '.
                             'FROM '.$CFG->prefix.'user u, '.
                             '     '.$CFG->prefix.'portfolio_attempts qa '.
                             'WHERE qa.qid = \''.$portfolioid.'\' AND '.
                             '      u.id = qa.userid');
    return ($users);
}

//////////////////////////////////////////////////////////////////////////////////////
/// Any other portfolio functions go here.  Each of them must have a name that 
/// starts with portfolio_

/// This function *really* shouldn't be needed, but since sometimes we can end up with
/// orphaned surveys, this will clean them up.
function portfolio_cleanup() {
    global $CFG;

    /// Find surveys that don't have portfolios associated with them.
    $sql = 'SELECT qs.* FROM '.$CFG->prefix.'portfolio_survey qs '.
           'LEFT JOIN '.$CFG->prefix.'portfolio q ON q.sid = qs.id '.
           'WHERE q.sid IS NULL';

    if ($surveys = get_records_sql($sql)) {
        foreach ($surveys as $survey) {
            portfolio_delete_survey($survey->id, 0);
        }
    }
    /// Find deleted questions and remove them from database (with their associated choices, etc. // TODO
    return true;
}

function portfolio_record_submission(&$portfolio, $userid, $rid=0) {
    $attempt['qid'] = $portfolio->id;
    $attempt['userid'] = $userid;
    $attempt['rid'] = $rid;
    $attempt['timemodified'] = time();
    return insert_record("portfolio_attempts", (object)$attempt, false);
}

function portfolio_delete_survey($sid, $qid) {
    global $portfolio_ARCHIVED;
/// Until backup is implemented, just mark the survey as archived.

    $status = true;

    /// Delete all responses for the survey:
    if ($responses = get_records('portfolio_response', 'survey_id', $sid, 'id')) {
        foreach ($responses as $response) {
            $status = $status && portfolio_delete_response($response->id);
        }
    }

    /// There really shouldn't be any more, but just to make sure...
    delete_records('portfolio_response', 'survey_id', $sid);
    delete_records('portfolio_attempts', 'qid', $qid);

    /// Delete all question data for the survey:
    if ($questions = get_records('portfolio_question', 'survey_id', $sid, 'id')) {
        foreach ($questions as $question) {
            delete_records('portfolio_question_choice', 'question_id', $question->id);
        }
        $status = $status && delete_records('portfolio_question', 'survey_id', $sid);
    }

    $status = $status && delete_records('portfolio_survey', 'id', $sid);

    return $status;
}

function portfolio_delete_response($rid) {

    $status = true;

    /// Delete all of the survey response data:
    delete_records('portfolio_response_bool', 'response_id', $rid);
    delete_records('portfolio_response_date', 'response_id', $rid);
    delete_records('portfolio_response_multiple', 'response_id', $rid);
    delete_records('portfolio_response_other', 'response_id', $rid);
    delete_records('portfolio_response_rank', 'response_id', $rid);
    delete_records('portfolio_response_single', 'response_id', $rid);
    delete_records('portfolio_response_text', 'response_id', $rid);

    $status = $status && delete_records('portfolio_response', 'id', $rid);
    $status = $status && delete_records('portfolio_attempts', 'rid', $rid);

    return $status;
}

function portfolio_get_active_surveys_menu() {
    global $portfolio_ACTIVE1;
    global $portfolio_ACTIVE2;

    $select = "status in ($portfolio_ACTIVE1,$portfolio_ACTIVE2)";
    return get_records_select_menu('portfolio_survey', $select);
}

function portfolio_get_surveys_menu($status=NULL) {

    $field = ($status) ? 'status' : $status;
    return get_records_menu('portfolio_survey', $field, $status);
}

/// Functions to call directly into phpESP.
/// Make sure a "require_once('phpESP/admin/phpESP.ini.php')" line is included.
/// Don't need to include this for all library functions, so don't.
function portfolio_get_survey_list($courseid=0, $type='') {
    global $portfolio_EDITING, $portfolio_ACTIVE1, $portfolio_ENDED,
           $portfolio_ARCHIVED, $portfolio_TESTING, $portfolio_ACTIVE2;

    if ($courseid == 0) {
        if (isadmin()) {
            $select = '';
            $fields = 'id,name,owner,realm,public,status';
        } else {
            return false;
        }
    } else if (!empty($type)) {
        if ($type == 'public') {
            $select = 'status != '.$portfolio_ARCHIVED.' AND realm = \''.$type.'\' ';
    /// Any survey owned by the user or typed as 'template' can be copied.
        } else if ($type == 'template') {
            $select = 'status != '.$portfolio_ARCHIVED.' AND '
                      .'(realm = \''.$type.'\' OR owner = \''.$courseid.'\') ';
        }
        $fields = 'id,name,owner,realm,status,title';
    } else {
        $select = 'status != '.$portfolio_ARCHIVED.' AND owner = \''.$courseid.'\' ';
        $fields = 'id,name,owner,realm,status';
    }
    return get_records_select('portfolio_survey', $select, 'realm,name', $fields);
}

function portfolio_survey_has_questions($sid) {
    return record_exists('portfolio_question', 'survey_id', $sid, 'deleted', 'N');
}

function portfolio_survey_exists($sid) {
    return record_exists('portfolio_survey', 'id', $sid);
}

function portfolio_print_survey_select($instance, $courseid=0, $sid=0, $fname='sid', $usenone=false, $type='') {
    global $CFG;

    if ($surveys = portfolio_get_survey_list($courseid, $type)) {
        $table->head[] = get_string('select');
        $table->head[] = get_string('name');
        $table->head[] = get_string('type', 'portfolio');
        $table->align = array('center', 'left', 'center');
        $table->size = array('*', '100%', '*');
        $table->wrap = array('', '', 'nowrap');

        if ($usenone) {
            $select = '<input type="radio" name="sid" value="0"'.(($sid == 0)?' checked':'').'>';
            $stat = '';
            $table->data[] = array($select, 'none', $stat);
        }

        $strpreview = get_string('preview');
        $strunknown = get_string('unknown', 'portfolio');
        $strpublic = get_string('public', 'portfolio');
        $strprivate = get_string('private', 'portfolio');
        $strtemplate = get_string('template', 'portfolio');
        $strinuse = get_string('inuse', 'portfolio');
        $strnotused = get_string('notused', 'portfolio');
        $strviewresp = get_string('viewresponses', 'portfolio');

        foreach ($surveys as $survey) {    
            if (empty($survey->realm)) {
                $stat = $strunknown;
            } else if ($survey->realm == 'public') {
                $stat = $strpublic;
            } else if ($survey->realm == 'private') {
                $stat = $strprivate;
            } else if ($survey->realm == 'template') {
                $stat = $strtemplate;
            } else {
                $stat = $strunknown;
            }
            // prevent creation of a new portfolio using a public portfolio IN THE SAME COURSE!
            if ($type == 'public' && $survey->owner == $courseid) {
            } else {
                $select = '<input type="radio" name="'.$fname.'" value="'.$type.'-'.$survey->id.'"'.
                          (($survey->id == $sid)?' checked':'').'>';
    
                $view = link_to_popup_window('/mod/portfolio/manage_survey.php?course='.$courseid.
                                             '&amp;qact=preview&amp;instance='.$instance.'&amp;sid='.$survey->id,
                                             $strpreview, $survey->title, 
                                             '', '', $strpreview, '', true);
                $table->data[] = array($select, $view, $stat);
            }
        }
        print_table($table);
        return true;
    } else {
        return false;
    }
}

function portfolio_get_type ($id) {
    switch ($id) {
    case 1:
        return get_string('yesno', 'portfolio');
    case 2:
        return get_string('textbox', 'portfolio');
    case 3:
        return get_string('essaybox', 'portfolio');
    case 4:
        return get_string('radiobuttons', 'portfolio');
    case 5:
        return get_string('checkboxes', 'portfolio');
    case 6:
        return get_string('dropdown', 'portfolio');
    case 8:
        return get_string('ratescale', 'portfolio');
    case 9:
        return get_string('date', 'portfolio');
    case 10:
        return get_string('numeric', 'portfolio');
    case 100:
        return get_string('sectiontext', 'portfolio');
    case 99:
        return '------------';
    default:
        return $id;                    
    }
}

function portfolio_get_view_actions() {
    return array('view','view all');
}

function portfolio_get_post_actions() {
    return array('submit','update');
}

?>