<?php // $Id: locallib.php,v 1.33.8.14 2007/11/27 17:49:05 joseph_rezeau Exp $

/**
 * This library replaces the phpESP application with Moodle specific code. It will eventually
 * replace all of the phpESP application, removing the dependency on that.
 */

/**
 * Updates the contents of the survey with the provided data. If no data is provided,
 * it checks for posted data.
 * 
 * @param int $survey_id The id of the survey to update.
 * @param string $old_tab The function that was being executed.
 * @param object $sdata The data to update the survey with.
 * 
 * @return string|boolean The function to go to, or false on error.
 * 
 */

/// Constants

// Allow phpESP to send email (BOOLEAN)
$ESPCONFIG['allow_email'] = true;

// Send human readable email, rather than machine readable (BOOLEAN)
$ESPCONFIG['human_email'] = false;

// Use authentication for designer interface (BOOLEAN)
$ESPCONFIG['auth_design'] = true;

// Use authentication for survey responders (BOOLEAN)
$ESPCONFIG['auth_response'] = true;

// Default number of option lines for new questions
$ESPCONFIG['default_num_choices'] = 10;

// Colors used by phpESP
$ESPCONFIG['main_bgcolor']      = '#FFFFFF';
$ESPCONFIG['link_color']        = '#0000CC';
$ESPCONFIG['vlink_color']       = '#0000CC';
$ESPCONFIG['alink_color']       = '#0000CC';
$ESPCONFIG['table_bgcolor']     = '#0099FF';
$ESPCONFIG['active_bgcolor']    = '#FFFFFF';
$ESPCONFIG['dim_bgcolor']       = '#3399CC';
$ESPCONFIG['error_color']       = '#FF0000';
$ESPCONFIG['warn_color']        = '#FF0000';
$ESPCONFIG['reqd_color']        = '#FF0000';
$ESPCONFIG['bgalt_color1']      = '#FFFFFF';
$ESPCONFIG['bgalt_color2']      = '#EEEEEE';

// phpESP css path
$ESPCONFIG['css_path'] = $CFG->dirroot.'/mod/portfolio/css/';


require_once('questiontypes/questiontypes.class.php');

define ('portfolioDEFAULTNUMCHOICES', 6); // JR instead of 10 (should be plenty)

class portfolio {

/// Class Properties
    /**
     * The survey record.
     * @var object $survey
     */
     var $survey;

/// Class Methods

    /**
     * The class constructor
     *
     */
    function portfolio($id = 0, $portfolio = null, &$course, &$cm, $addquestions = true) {

        if ($id) {
            $portfolio = get_record('portfolio', 'id', $id);
        }

        if (is_object($portfolio)) {
            $properties = get_object_vars($portfolio);
            foreach ($properties as $property => $value) {
                $this->$property = $value;
            }
        }

        $this->add_survey($this->sid);

        $this->course = $course;
        $this->cm = $cm;
//for 1.7        $this->context = get_context_instance(CONTEXT_MODULE, $cm->id);

        if ($addquestions) {
            $this->add_questions($this->sid);
        }

        $this->usehtmleditor = can_use_html_editor();
    }

    /**
     * Fake constructor to keep PHP5 happy
     *
     */
    function __construct($id = 0, $portfolio = null, &$course, &$cm, $addquestions = true) {
        $this->portfolio($id, $portfolio, $course, $cm, $addquestions);
    }

    /**
     * Adding a survey record to the object.
     *
     */
    function add_survey($sid = 0, $survey = null) {

        if ($sid) {
            $this->survey = get_record('portfolio_survey', 'id', $sid);
        } else if (is_object($survey)) {
            $this->survey = clone($survey);
        }
    }

    /**
     * Adding questions to the object.
     */
    function add_questions($sid = false, $section = false) {

        if ($sid === false) {
            $sid = $this->sid;
        }

        $select = 'survey_id = '.$sid.' AND deleted != \'Y\'';
        if ($records = get_records_select('portfolio_question', $select, 'position')) {
            $sec = 1;
            foreach ($records as $record) {
                $this->questions[$record->id] = new portfolio_question(0, $record);
//                if ($record->type_id != 100) {
                    if ($record->type_id != 99) {
                        $this->questionsbysec[$sec][$record->id] = &$this->questions[$record->id];
                    } else {
                        $sec++;
                    }
//                }
            }
        }
    }

    function view() {
        global $CFG, $USER,
            $portfolio_STUDENTVIEWRESPONSES_WHENANSWERED,
            $portfolio_STUDENTVIEWRESPONSES_WHENCLOSED,
            $portfolio_STUDENTVIEWRESPONSES_ALWAYS,
            $portfolio_TYPES;

        if ($this->course->category) {
            $id = $this->course->id;
            $shortname = $this->course->shortname;
            $navigation = "<a href=\"../../course/view.php?id=$id\">$shortname</a> ->";
        }
        $currentcss = '';
        if ( !empty($this->survey->theme) ) {
            $currentcss = '<link rel="stylesheet" type="text/css" href="'.
                $CFG->wwwroot.'/mod/portfolio/css/'.$this->survey->theme.'" />';
        }
        print_header_simple($this->name, "",
                     '<a href="index.php?id='.$this->course->id.'">'.$this->strportfolios.'</a> -> '.
                     $this->name, '', $currentcss, true, 
                     update_module_button($this->cm->id, $this->course->id, $this->strportfolio), 
                     navmenu($this->course, $this->cm));
        if (isguest() && !$this->allows_guests()) {
            echo('<br>');
            portfolio_notify(get_string("guestsnotallowed", "portfolio", $this->name));
            echo('<div class="returnLink"><a href="'.$CFG->wwwroot.'/course/view.php?id='.$this->course->id.'">'.
                get_string("continue").'</a></div>');            
            exit;
        }

        $strprint = get_string('print', 'portfolio');
        $strprinttooltip = get_string('printtooltip', 'portfolio');
        $strprintblank = get_string('printblank', 'portfolio');
        $strprintblanktooltip = get_string('printblanktooltip', 'portfolio');
        $blankportfolio = true;
        echo ('<div class="notice" style="float:right; margin:4px;"><div class="buttons">');        

        // open print friendly as popup window to fix XML strict bug contrib-200
            $linkname = get_string('printblank','portfolio');
            $height = ''; $width = '';
            $title = get_string('printblanktooltip','portfolio');
            $options= '';
            $name = '';
            $url = '/mod/portfolio/print.php?qid='.$this->id.'&amp;rid=0&amp;'.'courseid='.$this->course->id.'&amp;sec=1';
            $print = button_to_popup_window($url, $name, $linkname, $height, $width, $title, $options, false);
            echo($print);
        if (isstudent($this->course->id)) {
    
        $rid = $this->get_response($USER->id);
        $linkname = get_string('print','portfolio');
        $title = get_string('printtooltip','portfolio');
        $url = '/mod/portfolio/print.php?qid='.$this->id.'&amp;rid='.$rid.'&amp;courseid='.$this->course->id.'&amp;sec=1';
        $print = button_to_popup_window($url, '', $linkname,'' ,'' , $title, '', false);
        echo($print);           
        }
        if ($this->is_survey_owner() 
                && (($this->resp_view == $portfolio_STUDENTVIEWRESPONSES_WHENCLOSED && $this->is_closed() )
                    ||
                    ($this->resp_view == $portfolio_STUDENTVIEWRESPONSES_WHENANSWERED 
                        && !$this->user_can_take($USER->id)) )
                || isteacher($this->course->id)
                || $this->resp_view == $portfolio_STUDENTVIEWRESPONSES_ALWAYS) {

            button_to_popup_window('/mod/portfolio/manage_survey.php?qact=preview&amp;instance='.
                $this->id.'&amp;course='.$this->course->id.
                '&amp;sid='.$this->sid.'&amp;section=1', '', get_string('preview_label', 'portfolio'),0,0);            
            helpbutton("preview", get_string("preview_label", "portfolio"), "portfolio", true, false);


                $numresp = $this->count_submissions();
                $strviewresponses = get_string('viewresponses', 'portfolio', $numresp);
                $link = "report.php";
                $options = array();
                $options['instance'] = $this->id;
                $options['sid'] = $this->sid;
                $options['qact'] = 'vresp';
                $print = print_single_button($link, $options, $label=$strviewresponses, $method='get', $target='_self', $return=false);
            } else if (!$this->cm->visible) {
                notice(get_string("activityiscurrentlyhidden"));
            }

        echo('</div></div>');
    
    /// Print the main part of the page

        if (!$this->is_active()) {
            echo '<div class="message">'
            .get_string('notavail', 'portfolio')
            .'</div>';
        }
        else if (!$this->is_open()) {
            echo '<div class="message">'
            .get_string('notopen', 'portfolio', userdate($this->opendate))
            .'</div>';
        }
        else if ($this->is_closed()) {
            echo '<div class="message">'
            .get_string('closed', 'portfolio', userdate($this->closedate))
            .'</div>';
        }
        else if (!$this->user_is_eligible($USER->id)) {
            echo '<div class="message">'
            .get_string('noteligible', 'portfolio')
            .'</div>';
        }
        else if ($this->user_can_take($USER->id)) {
            $sid=$this->sid;    
            $quser = $USER->id;

            if ($this->survey->realm == 'template') {
                print_string('templatenotviewable', 'portfolio');
                print_footer($this->course);
                exit();
            }
            echo '<div class="box generalbox boxaligncenter boxwidthwide">';            
    
            $msg = $this->print_survey($USER->id, $quser);

            $viewform = data_submitted($CFG->wwwroot."/mod/portfolio/view.php");
    
    ///     If Survey was submitted with all required fields completed ($msg is empty),
    ///     then record the submittal.
            if (isset($viewform->submit) && isset($viewform->submittype) && 
                ($viewform->submittype == "Submit Survey") && empty($msg)) {

                if ($this->resume) {
                    $this->response_delete($viewform->rid, $viewform->sec);
                }
                
                $this->rid = $this->response_insert($this->survey->id, $viewform->sec, $viewform->rid, $quser, $viewform);
                $this->response_commit($this->rid);

                /// If it was a previous save, rid is in the form...
                if (!empty($viewform->rid) && is_numeric($viewform->rid)) {
                    $rid = $viewform->rid;

                /// Otherwise its in this object.
                } else {
                    $rid = $this->rid;
                }
                portfolio_record_submission($this, $USER->id, $rid);
                add_to_log($this->course->id, "portfolio", "submit", "view.php?id={$this->cm->id}", "{$this->name}", $this->cm->id, $USER->id);

                $this->response_send_email($this->rid);
                $this->response_goto_thankyou();
            }
                if ($this->resume) {
                }


            echo '</div>'; //div class="box generalbox boxaligncenter boxwidthwide"           
            
        } else {
            switch ($this->qtype) {
                case portfolioDAILY:
                    $msgstring = ' '.get_string('today', 'portfolio');
                    break;
                case portfolioWEEKLY:
                    $msgstring = ' '.get_string('thisweek', 'portfolio');
                    break;
                case portfolioMONTHLY:
                    $msgstring = ' '.get_string('thismonth', 'portfolio');
                    break;
                default:
                    $msgstring = '';
                    break;
            }
            echo ('<div class="message">'.get_string("alreadyfilled", "portfolio", $msgstring).'</div>');
        }
    
    /// Finish the page
        print_footer($this->course);
    }

   /**
    * Function to view an entire responses data.
    * 
    */
    function view_response($rid, $blankportfolio=false) {
        global $CFG;
        if ( !empty($this->survey->theme) ) {
            $href = $CFG->wwwroot.'/mod/portfolio/css/'.$this->survey->theme;
            echo '<script type="text/javascript" defer="defer">
                //<![CDATA[
                document.write("<link rel=\"stylesheet\" type=\"text/css\" href=\"'.$href.'\" />")
                //]]>
                </script>';     
        }

        print_simple_box_start();
        $this->print_survey_start('', 1, 1, 0, $rid);// JR

        $data = new Object();
        $i = 1;
        if (!$blankportfolio) {
            $this->response_import_all($rid, $data);
        }
        foreach ($this->questions as $question) {
            if ($question->type_id < portfolio_QUESPAGEBREAK) {
                $question->response_display($data, $i++);
            }
        }

        $this->print_survey_end(1, 1);
        print_simple_box_end();
    }

   /**
    * Function to view an entire responses data.
    * 
    */
    function view_all_responses($resps) {
        global $CFG, $QTYPENAMES;
        if ( !empty($this->survey->theme) ) {
            $href = $CFG->wwwroot.'/mod/portfolio/css/'.$this->survey->theme;
            echo '<script type="text/javascript" defer="defer">
                //<![CDATA[
                document.write("<link rel=\"stylesheet\" type=\"text/css\" href=\"'.$href.'\" />")
                //]]>
                </script>';     
        }
        print_simple_box_start();
        $this->print_survey_start('', 1, 1, 0);

        foreach ($resps as $resp) {
            $data[$resp->id] = new Object();
            $this->response_import_all($resp->id, $data[$resp->id]);
        }

        $i = 1;
        echo '<div class="mainTable">';
        foreach ($this->questions as $question) {
            if ($question->type_id < portfolio_QUESPAGEBREAK) {
                $method = $QTYPENAMES[$question->type_id].'_response_display';
                if (method_exists($question, $method)) {
                    $question->questionstart_survey_display($i);
                    foreach ($data as $respid => $respdata) {
                        echo '<div class="respdate">'.userdate($resps[$respid]->submitted).'</div>';
                        $question->$method($respdata);
                        echo '<hr />';
                    }
                    $question->questionend_survey_display($i);
                } else {
                    error(get_string('displaymethod', 'portfolio'));
                }
            $i++;
            }
        }
        echo '</div>';

        $this->print_survey_end(1, 1);
        print_simple_box_end();
    }

/// Access Methods
    function is_active() {
        return (!empty($this->survey));
    }
    
    function is_open() {
        return ($this->opendate > 0) ? ($this->opendate < time()) : true;
    }
    
    function is_closed() {
        return ($this->closedate > 0) ? ($this->closedate < time()) : false;
    }
    
    function user_can_take($userid) {
    
        if (!$this->is_active() || !$this->user_is_eligible($userid)) {
            return false;
        }
        else if ($this->qtype == portfolioUNLIMITED) {
            return true;
        }
        else if ($userid > 0){
            return $this->user_time_for_new_attempt($userid);
        }
        else {
            return false;
        }
    }
    
    function user_is_eligible($userid) {
        return ($this->resp_eligible == 'all') ||
               (($this->resp_eligible == 'students') && isstudent($this->course->id, $userid)) ||
               (($this->resp_eligible == 'teachers') && isteacher($this->course->id, $userid));
    }

    function user_time_for_new_attempt($userid) {

        $select = 'qid = '.$this->id.' AND userid = '.$userid;
        if (!($attempts = get_records_select('portfolio_attempts', $select, 'timemodified DESC'))) {
            return true;
        }

        $attempt = reset($attempts);
        $timenow = time();
        
        switch ($this->qtype) {

            case portfolioUNLIMITED:
                $cantake = true;
                break;

            case portfolioONCE:
                $cantake = false;
                break;

            case portfolioDAILY:
                $attemptyear = date('Y', $attempt->timemodified);
                $currentyear = date('Y', $timenow);
                $attemptdayofyear = date('z', $attempt->timemodified);
                $currentdayofyear = date('z', $timenow);
                $cantake = (($attemptyear < $currentyear) ||
                            (($attemptyear == $currentyear) && ($attemptdayofyear < $currentdayofyear)));
                break;

            case portfolioWEEKLY:
                $attemptyear = date('Y', $attempt->timemodified);
                $currentyear = date('Y', $timenow);
                $attemptweekofyear = date('W', $attempt->timemodified);
                $currentweekofyear = date('W', $timenow);
                $cantake = (($attemptyear < $currentyear) ||
                            (($attemptyear == $currentyear) && ($attemptweekofyear < $currentweekofyear)));
                break;

            case portfolioMONTHLY:
                $attemptyear = date('Y', $attempt->timemodified);
                $currentyear = date('Y', $timenow);
                $attemptmonthofyear = date('n', $attempt->timemodified);
                $currentmonthofyear = date('n', $timenow);
                $cantake = (($attemptyear < $currentyear) ||
                            (($attemptyear == $currentyear) && ($attemptmonthofyear < $currentmonthofyear)));
                break;

            default:
                $cantake = false;
                break;
        }

        return $cantake;
    }
    
    function is_survey_owner() {
        return ($this->course->id == $this->survey->owner);
    }
    
    function allows_guests() {
        return ($this->respondenttype == 'anonymous') && ($this->qtype == portfolioUNLIMITED) &&
               ($this->resp_eligible == 'all') && ($this->resume == 0);
    }

    function count_submissions($userid=false) {
        if (!$userid) {
            return count_records('portfolio_response', 'survey_id', $this->sid);
        } else {
            return count_records('portfolio_response', 'survey_id', $this->sid, 'username', $userid);
        }
    }

    function has_required($section = 0) {
        if (empty($this->questions)) {
            return false;
        } else if ($section <= 0) {
            foreach ($this->questions as $question) {
                if ($question->required == 'Y') {
                    return true;
                }
            }
        } else {
            foreach ($this->questionsbysec[$section] as $question) {
                if ($question->required == 'Y') {
                    return true;
                }
            }
        }
        return false;
    }
/// Display Methods

    function print_survey($userid=false, $quser) {
        global $USER, $PAGE, $CFG;
    
        if (!$userid) {
            $userid = $USER->id;
        }
    
        $formdata = data_submitted('nomatch');
        $formdata->rid = $this->get_response($quser);
        if (!empty($formdata->rid) && (empty($formdata->sec) || intval($formdata->sec) < 1)) {
            $formdata->sec = $this->response_select_max_sec($formdata->rid);
        }
        if (empty($formdata->sec)) {
            $formdata->sec = 1;
        } else {
            $formdata->sec = (intval($formdata->sec) > 0) ? intval($formdata->sec) : 1;
        }
    
        $num_sections = count($this->questionsbysec);    /// indexed by section.
        $msg = '';
        $action = $CFG->wwwroot.'/mod/portfolio/view.php?id='.$this->cm->id;

/// TODO - Need to rework this. Too much crossover with ->view method.
        if(!empty($formdata->submit)) {        
            $msg = $this->response_check_format($formdata->sec, $formdata);
            if(empty($msg)) {
                return;
            }
        }
    
        if(!empty($formdata->resume) && ($this->resume)) {
            $this->response_delete($formdata->rid, $formdata->sec);
            
            $formdata->rid = $this->response_insert($this->survey->id, $formdata->sec, $formdata->rid, $quser, $formdata);
            $this->response_commit($formdata->rid);
            $this->response_goto_saved($action);
        }
 // JR TODO save each section 's $formdata somewhere in case user returns to that page when navigating the portfolio...
        if(!empty($formdata->next)) {
            $this->response_delete($formdata->rid, $formdata->sec);
            $formdata->rid = $this->response_insert($this->survey->id, $formdata->sec, $formdata->rid, $quser, $formdata);
            $msg = $this->response_check_format($formdata->sec, $formdata);
            if ( $msg ) {
                $formdata->next = '';
            } else {
                $formdata->sec++;
            }
        }
        if (!empty($formdata->prev) && ($this->navigate)) {
            $this->response_delete($formdata->rid, $formdata->sec);
            $formdata->rid = $this->response_insert($this->survey->id, $formdata->sec, $formdata->rid, $quser, $formdata);
            $msg = $this->response_check_format($formdata->sec, $formdata);
            if ( $msg ) {
                $formdata->prev = '';
            } else {
                $formdata->sec--;
            }
        }
        if (!empty($formdata->rid)) {
            $this->response_import_sec($formdata->rid, $formdata->sec, $formdata);
        }
    
        echo '
    <script type="text/javascript">
    <!-- // Begin
    // when respondent enters text in !other field, corresponding radio button OR check box is automatically checked
    function other_check(name) {
      other = name.split("_");
      var f = document.phpesp_response;
      for (var i=0; i<=f.elements.length; i++) {
        if (f.elements[i].value == "other_"+other[1]) {
          f.elements[i].checked=true;
          break;
        }
      }
    }

    // function added by JR to automatically empty an !other text input field if another Radio button is clicked
    function other_check_empty(name, value) {
      var f = document.phpesp_response;
      for (var i=0; i<f.elements.length; i++) {
        if ((f.elements[i].name == name) && f.elements[i].value.substr(0,6) == "other_") {
            f.elements[i].checked=true;
            var otherid = f.elements[i].name + "_" + f.elements[i].value.substring(6);
            var other = document.getElementsByName (otherid);
            other[0].value = "";
            var actualbuttons = document.getElementsByName (name);
              for (var i=0; i<=actualbuttons.length; i++) {
                if (actualbuttons[i].value == value) {
                    actualbuttons[i].checked=true;
                    break;
                }
            }                      
        break;
        }
      }
    }

    // function added by JR in a Rate question type of sub-type Order to automatically uncheck a Radio button 
    // when another radio button in the same column is clicked
    function other_rate_uncheck(name, value) {
        col_name = name.substr(0, name.indexOf("_"));
        var inputbuttons = document.getElementsByTagName("input");
        for (var i=0; i<=inputbuttons.length - 1; i++) {
            button = inputbuttons[i];
            if (button.type == "radio" && button.name != name && button.value == value && button.name.substr(0, name.indexOf("_")) == col_name) {
                button.checked = false;
            }
        }        
    }

    // function added by JR to empty an !other text input when corresponding Check Box is clicked (supposedly to empty it)
    function checkbox_empty(name) {
        var actualbuttons = document.getElementsByName (name);
        for (var i=0; i<=actualbuttons.length; i++) {
            if (actualbuttons[i].value.substr(0,6) == "other_") {
                name = name.substring(0,name.length-2) + actualbuttons[i].value.substring(5); 
                var othertext = document.getElementsByName (name);
                othertext[0].value = "";
                break;
            }
        }
    }
    // End -->
    </script>
            ';

    ?>
    <form method="post" action="<?php echo($action); ?>">
    <div>
    <input type="hidden" name="referer" value="<?php echo (!empty($formdata->referer) ? htmlspecialchars($formdata->referer) : ''); ?>" />
    <input type="hidden" name="a" value="<?php echo($this->id); ?>" />
    <input type="hidden" name="sid" value="<?php echo($this->survey->id); ?>" />
    <input type="hidden" name="rid" value="<?php echo (isset($formdata->rid) ? $formdata->rid : '0'); ?>" />
    <input type="hidden" name="sec" value="<?php echo($formdata->sec); ?>" />
    </div>
    <?php
        $this->survey_render($formdata->sec, $msg, $formdata);
        echo '<div class="notice" style="float:left"><div class="buttons">';        
        if (($this->navigate) && ($formdata->sec > 1)) {
            echo '<input type="submit" name="prev" value="'.get_string('previouspage', 'portfolio').'" />';
        }
        if ($this->resume) {
            echo '<input type="submit" name="resume" value="'.get_string('save', 'portfolio').'" />';
        }
    //  Add a 'hidden' variable for the mod's 'view.php', and use a language variable for the submit button.
        if($formdata->sec == $num_sections) {
/*            echo '
        <div><input type="hidden" name="submittype" value="Submit Survey" />
        <input type="submit" name="submit" value="'.get_string('submitsurvey', 'portfolio').'" /></div>';*/
        } else {
            echo '<div><input type="submit" name="next" value="'.get_string('nextpage', 'portfolio').'" /></div>';
        }
        echo '</div></div>'; //divs notice & buttons
        echo '</form>';

        return $msg;
    }
    
    function survey_render($section = 1, $message = '', &$formdata) {

        $usehtmleditor = can_use_html_editor();
        $this->usehtmleditor = null;
    
        if(empty($section)) {
            $section = 1;
        }
    
        $has_choices = $this->type_has_choices();
   
        $num_sections = count($this->questionsbysec);    /// indexed by section.
        if($section > $num_sections) {
            return(false);  // invalid section
        }
  
    // check to see if there are required questions
        $has_required = $this->has_required($section);
    
    // find out what question number we are on $i
        $i = 1;
        for($j = 2; $j<=$section; $j++) {
            $i += count($this->questionsbysec[$j-1]);
        }

        $this->print_survey_start($message, $section, $num_sections, $has_required);

        foreach ($this->questionsbysec[$section] as $question) {
            if ($question->type === 'Essay Box') {
                $this->usehtmleditor = can_use_html_editor();
            }
            $question->survey_display($formdata, $i++, $this->usehtmleditor);
            /// Bug MDL-7292 - Don't count section text as a question number.
            if ($question->type_id == portfolio_QUESSECTIONTEXT) {
                $i--;
            }
            // process each question
        }
        // end of questions
        echo ('<div class="surveyPage">');    
        $this->print_survey_end($section, $num_sections);
        echo '</div>';
        return;
    }
    
    function print_survey_start($message, $section, $num_sections, $has_required, $rid='') {
        $userid = '';
        $resp = '';
        $groupname = '';
        $timesubmitted = '';
        if ($rid) {
            $courseid = $this->course->id;
            if ($resp = get_record('portfolio_response', 'id', $rid) ) {
                if ($this->respondenttype == 'fullname') {
                    $userid = $resp->username;
                    // display name of group that student belongs to - if more than one, display first group + etc.
                    if ($groups = get_groups($courseid, $resp->username)) {
                        $group = current($groups);
                        if (count($groups) == 1) {
                            $groupname = ' ('.get_string('group').': '.$group->name.')';
                        } else {
                            $groupname = ' ('.get_string('groups').': '.$group->name.', etc. )';                        
                        }
                    } else {
                        $groupname = ' ('.get_string('no') . ' ' . get_string('group').')';
                    }
                }
            }
        }
        $ruser = '';
        if ($resp) {
            if ($user = get_record('user', 'id', $userid)) {
                $ruser = fullname($user);
            }            
            if ($this->respondenttype == 'anonymous') {
                $ruser = '- '.get_string('anonymous', 'portfolio').' -';            
            } else {
            // JR DEV comment following line out if you do NOT want time submitted displayed in Anonymous surveys
                $timesubmitted = '&nbsp;'.get_string('submitted', 'portfolio').'&nbsp;'.userdate($resp->submitted);
            }
        }
        if ($ruser) {
                    echo ('<strong>'.get_string('respondent', 'portfolio').': '.$ruser.'</strong>');
                    echo ($groupname);
                    echo ($timesubmitted);
        }
        echo '<h2 class="surveyTitle">'.s($this->survey->title).'</h2>';
        if ($section == 1) {
                if ($this->survey->subtitle) {
                    echo '<h3 class="surveySubtitle">'.s($this->survey->subtitle).'</h3>';
                }
                if ($this->survey->info) {
                    echo '<div class="addInfo">'.format_text($this->survey->info, FORMAT_HTML).'</div>';
                }
        }
        if($num_sections>1) {
            $a = '';
            $a->page = $section;
            $a->totpages = $num_sections;
            echo '<div class="surveyPage">&nbsp;'.get_string('pageof', 'portfolio', $a).'</div>';
        }
        if ($message) {
            echo '<div class="message">'.$message.'</div>'; //JR
        }

    }
    
    function print_survey_end($section, $num_sections) {
        if($num_sections>1) {
            $a = '';
            $a->page = $section;
            $a->totpages = $num_sections;
            echo get_string('pageof', 'portfolio', $a).'&nbsp;&nbsp;';
        }
    }
    
    function survey_print_render($message = '', $referer='', $courseid) {
        global $USER;
        if (! $course = get_record("course", "id", $courseid)) {
            error(get_string('incorrectcourseid', 'portfolio'));
        }
        $this->course = $course;
        $text_input_add = ' readonly="true"';
        $radio_input_add = ' onclick="this.checked=this.defaultChecked;"';
        $sid = $this->sid;
        $usehtmleditor = can_use_html_editor();
        $formdata = data_submitted($referer);

        if ($this->resume && !isset($formdata) && empty($formdata->rid) ) {
            $formdata->rid = $this->get_response($USER->id, $formdata->rid);
        }

        if (!empty($formdata->rid)) {
        // If we're viewing a response, use this method.
            $this->view_response($formdata->rid);
            return;
        }
    
        if(empty($section)) {
            $section = 1;
        }
    
        $has_choices = $this->type_has_choices();
    
        $num_sections = count($this->questionsbysec);
        if($section > $num_sections)
            return(false);  // invalid section
    
        $has_required = $this->has_required();
    
    // find out what question number we are on $i
        $i = 1;
        for($j = 2; $j<=$section; $j++) {
            $i += count($this->questionsbysec[$j-1]);
        }

        print_simple_box_start();
        $this->print_survey_start($message, 1, 1, $has_required);
        /// Print all sections:
        foreach ($this->questionsbysec as $section) {
            foreach ($section as $question) {
                $question->survey_display($formdata, $i++, $usehtmleditor=null);
            }
        }
        // end of questions
    
        print_simple_box_end();

        echo '</form></div>';
        return;
    }

    function survey_update($old_tab, $sdata, $newquestion) {
        global $CFG, $SESSION;
    
        $errstr = '';
    
        // do not need update
        if(empty($old_tab)) {
            return(false);
        }
    
        if (empty($sdata)) {
            $sdata = data_submitted('nomatch');
        }
    
        $f_arr = array();
        $v_arr = array();
    
        // new survey
        if(empty($this->survey->id)) {
            if (isset($sdata->name)) {
            $sdata->name = eregi_replace(
                "[^A-Z0-9]+", "_", trim($sdata->name) );
            $sdata->name = ereg_replace('_$',"",$sdata->name);
            }
    
            // need to fill out at least some info on 1st tab before proceeding
            if(empty($sdata->name) || empty($sdata->title) || empty($sdata->realm)) {
                $tab = "general";
                $errstr = get_string('errrequiredfields', 'portfolio');
                return(false);
            }
    
            // create a new survey in the database
            $fields = array('name','realm','title','subtitle','email','theme','thanks_page','thank_head','thank_body','info');
            $record = new Object();
            $record->id = 0;
            $record->owner = $sdata->owner;
            foreach($fields as $f) {
                if(isset($sdata->$f)) {
                    $record->$f = $sdata->$f;
                }
            }

            $this->survey->id = insert_record('portfolio_survey', $record);
            $this->add_survey($this->survey->id);
    
            if(!$this->survey->id) {
                $tab = "general";
                $errstr = get_string('errnewname', 'portfolio') .' [ :  ]';
                return(false);
            }
    
            return($this->survey->id);
        }
    
        // survey already started
        switch($old_tab) {
            // coming from the general tab ...
            case "general":
                if (isset($sdata->name)) {
                $sdata->name = eregi_replace(
                    "[^A-Z0-9]+", "_", trim($sdata->name) );
                $sdata->name = ereg_replace('_$',"",$sdata->name);
                }
    
                if(empty($sdata->name) || empty($sdata->title)
                        || empty($sdata->realm)) {
                    $errstr = get_string('errrequiredfields', 'portfolio');
                    return(false);
                }
    
                $fields = array('name','realm','title','subtitle','email','theme','thanks_page','thank_head','thank_body','info');
    
                $name = get_field('portfolio_survey', 'name', 'id', $this->survey->id);
    
                // trying to change survey name
                if(trim($name) != trim($sdata->name)) {
                    $count = count_records('portfolio_survey', 'name', $sdata->name);
                    if($count != 0) {
                        $errstr = get_string('errnewname', 'portfolio');
                        return(false);
                    }
                }
    
                // UPDATE the row in the DB with current values
                $survey_record = new Object();
                $survey_record->id = $this->survey->id;
                foreach($fields as $f) {
                    $survey_record->$f = trim($sdata->$f);
                }
    
                $result = update_record('portfolio_survey', $survey_record);
                if(!$result) {
                    $errstr = get_string('warning', 'portfolio').' [ :  ]';
                    return(false);
                }
                return($this->survey->id);
    
            // coming from the questions tab ...
            case "questions":
            // if extra choices come from a new question being edited, save some parameters and break
                if (isset($sdata->extra_choices)) {
                    if ($sdata->extra_choices && !$sdata->id) {
                        $SESSION->NEWQUESTION = true;
                        $formdata->content= $sdata->content;
                        break;
                    }
                }
                // initialize error messages to be transmitted via SESSION variables to questions.php & questions_options.php
                $SESSION->ERROR_QUESTIONTEXT = '';
                $SESSION->ERROR_QUESTIONOPTION = '';
                $SESSION->ERROR_QUESTIONTYPE = '';
                $SESSION->NEWQUESTION = false;
                $newquestion = false;
                if (isset($sdata->q)) {
                    if ($sdata->q == get_string('newfieldenter', 'portfolio')) {
                        $newquestion = true;
                        $SESSION->NEWQUESTION = true;
                    }
                }
                
                if (isset($sdata->content)) {
                    if(!$newquestion && empty($sdata->content) && empty($sdata->name)) {
                        return($this->survey->id);
                    }
                }
                if(!$sdata->content) {
                    $dont_clear = 1;
                    $SESSION->ERROR_QUESTIONTEXT = get_string('errquestiontext', 'portfolio');
                    return(false);
                }
    
                // constraint: question type required
                if($sdata->type_id == 0) {
                    $dont_clear = 1;
                    $SESSION->ERROR_QUESTIONTYPE = get_string('errquestiontype', 'portfolio');
                    return(false);
                }
    
                // constraint: qid must be int or empty
                if($sdata->id == get_string('newfield', 'portfolio')) {
                    $qid = '';
                }
                $qid = intval($sdata->id);   # curr_q_id
                $has_choices = $this->type_has_choices();
               
                if($has_choices[$sdata->type_id]) {

// this error trap works only half ... check what happens next
                }
                // constraint: can not change between question w/ answer choices and one w/o
                if(!empty($qid)) {
                    $old_type_id = get_field('portfolio_question', 'type_id', 'survey_id', $this->survey->id, 'id', $qid);
                    if($has_choices[$sdata->type_id] != $has_choices[$old_type_id]) { // trying to change between incompatible question types
                        $tab = "questions";
                        $sdata->type_id = $old_type_id;
                        $dont_clear = 1;
                    $SESSION->ERROR_QUESTIONTYPE = get_string('errquestionchange', 'portfolio');
//error('ERROR CANNOT CHANGE TYPE');
                        return(false);
                    }
                }
    
                // constraint: length must be int
                $sdata->length  = intval($sdata->length) or 0;
    
                // constraint: precise must be int
                $sdata->precise = intval($sdata->precise) or 0;

                // default = blank space for choice_1 in rate question type
                if ($sdata->type_id == 8) {
                    if (!isset($sdata->choice_content_1) || $sdata->choice_content_1 == '') {
                        $sdata->choice_content_1 = ''; // ascii 149
                    }
                }    

                // defaults for length field
                if(empty($sdata->length) && $sdata->type_id < 50) {
                    $arr = array(
                        0,      // 0: unused
                        0,      // 1: Yes/No
                        20,     // 2: Text Box  (width)
                        0,     // 3: Essay Box 0 = use HTML editor; > 0 use plain text (width)
                        0,      // 4: Radio Buttons
                        0,      // 5: Check Boxes (minumum)
                        0,      // 6: Dropdown Box (length)
                        5,      // 7: Rating (# cols)
                        5,      // 8: Rate (# cols)
                        0,      // 9: Date
                        10      // 10: Numeric (digits)
                        );
                    $sdata->length = $arr[$sdata->type_id];
                }
    
                // defaults for precision field
                if(empty($sdata->precise) && $sdata->type_id < 50) {
                    $arr = array(
                        0,      // 0: unused
                        0,      // 1: Yes/No
                        20,      // 2: Text Box (maximum text length)
                        0,      // 3: Essay Box 0 = use HTML editor; > 0 use plain text (height)
                        0,     // 4: Radio Buttons // changed from 10 to 0 by JR
                        0,      // 5: Check Boxes (maximum)
                        0,      // 6: Dropdown Box
                        0,      // 7: Rating (use N/A)
                        0,      // 8: Rate (use N/A)
                        0,      // 9: Date
                        0       // 10: Numeric (decimal)
                        );
                    $sdata->precise = $arr[$sdata->type_id];
                }
    
                // UPDATE row in the DB for the current question
                if($qid != '') {
                    $fields = array('name','type_id','length','precise','required','content');
                    $question_record = new Object();
                    $question_record->id = $qid;
                    foreach($fields as $f) {
                        if(isset($sdata->$f))
                            $question_record->$f = trim($sdata->$f);
                    }
                    $result = update_record('portfolio_question', $question_record);
    
                // INSERT row in the DB for new question
                } else {
                    // set the position to the end
                    $sql = 'SELECT MAX(position) as maxpos FROM '.$CFG->prefix.'portfolio_question '.
                           'WHERE survey_id = '.$this->survey->id;
                    if ($record = get_record_sql($sql)) {
                        $sdata->position = $record->maxpos + 1;
                    } else {
                        $sdata->position = 1;
                    }
    
                    $sdata->survey_id = $this->survey->id;
                    $fields = array('survey_id','name','type_id','length','precise','required','content','position');
                    $question_record = new Object();
                    foreach($fields as $f) {
                        if(isset($sdata->$f)) {
                            $question_record->$f = trim($sdata->$f);
                        }
                    }
                    $result = insert_record('portfolio_question', $question_record);
                }
                if($qid == '')
                    $qid = $result;
                $sdata->id = $qid;
                if(!$result) {
                    $dont_clear = 1;
                    $errstr = get_string('warning', 'portfolio').' [ :  ]';
                    return(false);
                }
    
                // UPDATE or INSERT rows for each of the question choices for this question
                if($has_choices[$sdata->type_id]) {
                    $cids = array();
                    $sql = 'SELECT c.id, c.question_id FROM '.$CFG->prefix.'portfolio_question q, '.$CFG->prefix.'portfolio_question_choice c WHERE q.id=c.question_id AND q.survey_id='.$this->survey->id;
                    if (!($records = get_records_sql($sql))) {
                        $records = array();
                    }
                    foreach ($records as $record) {
                        array_push($cids, $record->id);
                    }
                    $count = 0;
                    for($i = 1; $i < $sdata->num_choices+1; $i++) {
                        $choice_id      = intval($sdata->{"choice_id_${i}"});
                        $choice_content = trim($sdata->{"choice_content_${i}"});
                        // each of the submitted choices
                        if($choice_id=='' && $choice_content!='') {
                            // insert (new)
                            $choice_record = new Object();
                            $choice_record->question_id = $qid;
                            $choice_record->content = $choice_content;
                            $result = insert_record('portfolio_question_choice', $choice_record);
                            ++$count;
                        } elseif($choice_id!='' && $choice_content=='') {
                            // delete (old)
                            $result = delete_records('portfolio_question_choice', 'id', $choice_id);
                        } elseif($choice_id!='' && $choice_content!='' && in_array($choice_id, $cids)) {
                            // update (old)
                            $result = set_field('portfolio_question_choice', 'content', $choice_content, 'id', $choice_id);
                            ++$count;
                        }
                        if($sql != '') {
                            if(!$result) {
                                $dont_clear = 1;
                                $errstr = get_string('warning', 'portfolio').' [ :  ]';
                                return(false);
                            }
                        }
                    }
// TODO check to be finalized JR
/*
                    if($count < 2 && !isset($sdata->extra_choices)) {
                        $dont_clear = 1;
                        switch ($sdata->type_id) {
                            case 4:
                                $type = get_string('radiobuttons', 'portfolio');
                                break;
                            case 5:
                                $type = get_string('checkboxes', 'portfolio');
                        }
                        if (isset($formdata->q)) {
                        $errstr = get_string('errquestionoption', 'portfolio', $type);
                        $SESSION->ERROR_QUESTIONOPTION = $errstr.$SESSION->portfolio_curr_q    ;
                        return(false);
                    }
*/
                }
                return($this->survey->id);
    
            case "preview":
                // can not change anything here yet, so no need to update DB.
                return($this->survey->id);
    
            case "order":
                // it updates the DB itself
                return($this->survey->id);
        }
        return($this->survey->id);
    }
    
    /* Creates an editable copy of a survey. */
    function survey_copy($owner) {

        // clear the sid, clear the creation date, change the name, and clear the status
        // Since we're copying a data record, addslashes.
        $survey = addslashes_object($this->survey);

        unset($survey->id);
        $survey->changed = time();
        $survey->owner = $owner;
        $survey->name .= '_copy';
        $survey->status = 0;
    
        // check for 'name' conflict, and resolve
        $i=0;
        $name = $survey->name;
        while (count_records('portfolio_survey', 'name', $name) > 0) {
            $name = $survey->name.(++$i);
        }
        if($i) {
            $survey->name .= $i;
        }
    
        // create new survey
        if (!($new_sid = insert_record('portfolio_survey', $survey))) {
            return(false);
        }

        // make copies of all the questions
        $pos=1;
        foreach ($this->questions as $question) {
            $tid = $question->type_id;
            $qid = $question->id;
    
            // fix some fields first
            unset($question->id);
            $question->survey_id = $new_sid;
            $question->position = $pos++;
            $question->name = addslashes($question->name);
            $question->content = addslashes($question->content);
    
            // copy question to new survey
            if (!($new_qid = insert_record('portfolio_question', $question))) {
                return(false);
            }
    
            foreach ($question->choices as $choice) {
                unset($choice->id);
                $choice->question_id = $new_qid;
                $choice->content = addslashes($choice->content);
                $choice->value = addslashes($choice->value);
                if (!insert_record('portfolio_question_choice', $choice)) {
                    return(false);
                }
            }
        }
    
        return($new_sid);
    }

    function type_has_choices() {
        $has_choices = array();
    
        if ($records = get_records('portfolio_question_type', '', '', 'id', 'id,has_choices')) {
            foreach ($records as $record) {
                if($record->has_choices == 'Y') {
                    $has_choices[$record->id]=1;
                } else {
                    $has_choices[$record->id]=0;
                }
            }
        } else {
            $has_choices = array();
        }
    
        return($has_choices);
    }
    
    function array_to_insql($array) {
        if (count($array))
            return("IN (".ereg_replace("([^,]+)","'\\1'",join(",",$array)).")");
        return 'IS NULL';
    }
    
    // ---- RESPONSE LIBRARY
    
    function response_check_format($section, &$formdata, $qnum='') {
        $missing = 0;
        $strmissing = ''; // missing questions
        $wrongformat = 0;
        $strwrongformat = ''; // wrongly formatted questions (Numeric, 5:Check Boxes, Date)
        $i = 1;
        for($j = 2; $j<=$section; $j++) {
            $i += count($this->questionsbysec[$j-1]);
        }
        $qnum = $i - 1;

        foreach ($this->questionsbysec[$section] as $record) {

            $qid = $record->id;
            $tid = $record->type_id;
            $lid = $record->length;
            $pid = $record->precise;
            $content = $record->content;
            if ($tid != 100) {
                $qnum++;                
            }
            if ( ($record->required == 'Y') && ($record->deleted == 'N') && ((isset($formdata->{'q'.$qid}) && $formdata->{'q'.$qid} == '') || (!isset($formdata->{'q'.$qid}))) && $tid != 8 && $tid != 100 ) {
                $missing++;
                $strmissing .= get_string('num', 'portfolio').$qnum.'. ';
            }

            switch ($tid) {

            case 4: // Radio Buttons with !other field
                if (!isset($formdata->{'q'.$qid})) {
                    break;
                }
                $resp = $formdata->{'q'.$qid};
                $pos = strpos($resp, 'other_');
                if (is_int($pos) == true && $record->required == 'Y') {
                    $resp = 'q'.$qid.''.substr($resp,5);
                    if (!$formdata->$resp) {
                        $missing++;
                        $strmissing .= get_string('num', 'portfolio').$qnum.'. ';
                    }        
                }
                break;

            case 5: // Check Boxes
                if (!isset($formdata->{'q'.$qid})) {
                    break;
                }
                $resps = $formdata->{'q'.$qid};
                $nbrespchoices = count($resps);
                $nbquestchoices = count($record->choices); 
                $min = $lid;
                $max = $pid;
                if ($max == 0) {
                    $max = $nbquestchoices;
                }
                if ($min > $max) {
                    $min = $max; // sanity check
                }
                $min = min($nbquestchoices, $min);
                // number of ticked boxes is not within min and max set limits
                if ( $nbrespchoices && ($nbrespchoices < $min || $nbrespchoices > $max) ) {
                    $wrongformat++;
                    $strwrongformat .= get_string('num', 'portfolio').$qnum.'. ';
                    break;
                }
                break;

            case 6: // Drop
                $resp = $formdata->{'q'.$qid};
                if (!$resp && $record->required == 'Y') {
                    $missing++;
                    $strmissing .= get_string('num', 'portfolio').$qnum.'. ';
                }
                break;
            
            case 8: // Rate
                $num = 0;
                $nbchoices = count($record->choices);
                foreach ($record->choices as $cid => $choice) {
                    // in case we have named degrees on the Likert scale, count them to substract from nbchoices
                    $nameddegrees = 0;
                    $content = $choice->content;
                    if (strpos($content,'=',1) && is_numeric(substr($content,0,1))) {
                        $nameddegrees++;
                    } else {
                        $str = 'q'."{$record->id}_$cid";
                        for ($j = 0; $j < $record->length; $j++) {
                            $num += (isset($formdata->$str) && ($j == $formdata->$str));
                        }
                        $num += (($record->precise) && isset($formdata->$str) && ($formdata->$str == -1));
                    }
                    $nbchoices -= $nameddegrees;
                }
                if ( $num == 0 && $record->required == 'Y') {
                    $missing++;
                    $strmissing .= get_string('num', 'portfolio').$qnum.'. ';
                    break;
                }
                if ( ($num != $nbchoices) && ($num!=0) ) {
                    $wrongformat++;
                    $strwrongformat .= get_string('num', 'portfolio').$qnum.'. ';
                }
                break;

            case 9: // Date
                if (($formdata->{'q'.$qid} != '') && (!portfolio_check_date($formdata->{'q'.$qid})) ) {
                    $wrongformat++;
                    $strwrongformat .= get_string('num', 'portfolio').$qnum.'. ';
                }
                break;

            case 10: // Numeric
                if ( ($formdata->{'q'.$qid} != '') && (!is_numeric($formdata->{'q'.$qid})) ) {
                    $wrongformat++;
                    $strwrongformat .= get_string('num', 'portfolio').$qnum.'. ';
                }
                break;

            default: 
                break;
            }
        }
        $message ='';
        if($missing) {
            if ($missing == 1) {
                $message = get_string('missingquestion', 'portfolio').$strmissing;
            } else {
                $message = get_string('missingquestions', 'portfolio').$strmissing;
            }
            if ($wrongformat) {
                $message .= '<br />';
            }
        }
        if($wrongformat) {
            if ($wrongformat == 1) {
                $message .= get_string('wrongformat', 'portfolio').$strwrongformat;
            } else {
                $message .= get_string('wrongformats', 'portfolio').$strwrongformat;
            }
        }
        return ($message);
    }

    
    function response_delete($rid, $sec = null) {
        if (empty($rid)) {
            return;
        }
    
        if ($sec != null) {
            if ($sec < 1) {
                return;
            }
    
            /* get question_id's in this section */
            $qids = '';
            foreach ($this->questionsbysec[$sec] as $question) {
                if (empty($qids)) {
                    $qids .= ' AND question_id IN ('.$question->id;
                } else {
                    $qids .= ','.$question->id;
                }
            }
            if (!empty($qids)) {
                $qids .= ')';
            } else {
                return;
            }
        } else {
            /* delete all */
            $qids = '';
        }
    
        /* delete values */
        $select = 'response_id = \''.$rid.'\' '.$qids;
        foreach (array('bool', 'single', 'multiple', 'rank', 'text', 'other', 'date') as $tbl) {
            delete_records_select('portfolio_response_'.$tbl, $select);
        }
    }
    
    function response_import_sec($rid, $sec, &$varr) {

        if ($sec < 1 || !isset($this->questionsbysec[$sec])) {
            return;
        }
        /* get question_id's in this section */
        $ids = array();
        foreach ($this->questionsbysec[$sec] as $question) {
            $ids[] = $question->id;
        }

        $vals = $this->response_select($rid, 'content', $ids);
    
        reset($vals);
        foreach ($vals as $id => $arr) {
            if (isset($arr[0]) && is_array($arr[0])) {
                // multiple
                $varr->{'q'.$id} = array_map('array_pop', $arr);
            } else {
                $varr->{'q'.$id} = array_pop($arr);
            }
        }    
    }
    
    function response_import_all($rid, &$varr) {

        /* get question_id's in this section */
        $ids = array();
        foreach ($this->questions as $question) {
            $ids[] = $question->id;
        }

        $vals = $this->response_select($rid, 'content', $ids);
    
        reset($vals);
        foreach ($vals as $id => $arr) {
            if (isset($arr[0]) && is_array($arr[0])) {
                // multiple
                $varr->{'q'.$id} = array_map('array_pop', $arr);
            } else {
                $varr->{'q'.$id} = array_pop($arr);
            }
        }
    }
    
    function response_commit($rid) {
        $record = new object;
        $record->id = $rid;
        $record->complete = 'N';
        $record->submitted = time();
        return update_record('portfolio_response', $record);
    }
    
    function get_response($username, $rid = 0) {
        $rid = intval($rid);
        if ($rid != 0) {
            // check for valid rid
            $fields = 'id, username';
            $select = 'id = '.$rid.' AND survey_id = '.$this->sid.' AND username = \''.$username.'\' AND complete = \'N\'';
            return (get_record_select('portfolio_response', $select, $fields) !== false) ? $rid : '';
    
        } else {
            // find latest in progress rid
            $select = 'survey_id = '.$this->sid.' AND complete = \'N\' AND username = \''.$username.'\'';
            if ($records = get_records_select('portfolio_response', $select, 'submitted DESC',
                                              'id,survey_id', 0, 1)) {
                $rec = reset($records);
                return $rec->id;
            } else {
                return '';
            }
        }
    }
    
    function response_select_max_sec($rid) {
        $pos = $this->response_select_max_pos($rid);
        $select = 'survey_id = \''.$this->sid.'\' AND type_id = 99 AND position < '.$pos.' AND deleted = \'N\'';
        $max = count_records_select('portfolio_question', $select) + 1;
    
        return $max;
    }
    
    function response_select_max_pos($rid) {
        $max = 0;
        
        global $CFG;
        foreach (array('bool', 'single', 'multiple', 'rank', 'text', 'other', 'date') as $tbl) {
            $sql = 'SELECT MAX(q.position) as num FROM '.$CFG->prefix.'portfolio_response_'.$tbl.' a, '.
                                                         $CFG->prefix.'portfolio_question q '.
                   'WHERE a.response_id = \''.$rid.'\' AND '.
                   'q.id = a.question_id AND '.
                   'q.survey_id = \''.$this->sid.'\' AND '.
                   'q.deleted = \'N\'';
            if ($record = get_record_sql($sql)) {
                $max = (int)$record->num;
            }
        }
        return $max;
    }
    
/* {{{ proto array response_select_name(int survey_id, int response_id, array question_ids)
   A wrapper around response_select(), that returns an array of
   key/value pairs using the field name as the key.
 */
    function response_select_name($rid, $qids = null) {
        $res = $this->response_select($rid, 'position,type_id,name', $qids);
        $nam = array();
        reset($res);
        $num = 1;
        $subqnum = 0;
        $oldpos = '';
        while(list($qid, $arr) = each($res)) {
            $key = null;
            $noname = false;
            if (isset($arr[2]) && $arr[2] == '') {
                $arr[2] = '_qu_'.sprintf("%02d", $arr[0]);
                $noname = true;
            }
            if (strstr($qid, '_')) {
                // rank or other
                if ($arr[1] != 8)
                    continue; // other

                // rank
                $str1 = $arr[3];
                if ($oldpos != $arr[0]) {
                    $subqnum = 1;
                    $oldpos = $arr[0];
                } else {
                        $subqnum++;                
                }
                if ($noname ) {  
                    $str1 = $subqnum;
                }
                $arr[2] .= "->$str1";
                if ($arr[4] == -1) {
                    $arr[4] = get_string('notapplicable', 'portfolio');
                } else {
                    $arr[4]++;           
                }
                $nam[$arr[2]] = $arr[4];
                   
                continue;
            }
            if (is_array($arr[0])) {
                // multiple

                $key = $arr[0][2];
                if ($key == '') {
                    $key = '_qu_'.sprintf("%02d", $arr[0][0]);
                }
                $val = array();
                foreach ($arr as $subarr) {
                    if (ereg("^!other", $subarr[3])) {
                        $tmpv = preg_replace(array("/^!other=/","/^!other/"),
                                array('', 'Other'), $subarr[3]);
                        $tmp = preg_replace("/^other/", $qid, $subarr[4]);
                        if (isset($res[$tmp]))
                            $tmpv .= ': '. $res[$tmp][2];
                        array_push($val, $tmpv);
                    } else {
                        if ($subarr[3] !='_') {
                            array_push($val, $subarr[3]);
                        }
                    }
                }
            } else {
                $key = $arr[2];
                if (ereg("^!other", $arr[3])) {
                    $val = preg_replace(array("/^!other=/","/^!other/"),
                            array('', 'Other'), $arr[3]);
                    $tmp = preg_replace("/^other/", $qid, $arr[4]);
                    if (isset($res[$tmp]))
                        $val .= ': '. $res[$tmp][2];
                } else {
                    $val = $arr[3];
                }
            }
                            $subqnum = 0;
            
            if (array_key_exists($key, $nam)){
                $key = $key.$num;
            }
            $nam[$key] = $val;
            $num++;
        }
        return $nam;
    }

    function response_send_email($rid, $userid=false) {
        global $CFG, $USER;
        require_once($CFG->libdir.'/phpmailer/class.phpmailer.php');

        $response = get_record('portfolio_response', 'id', $rid);
        
        if ($record = get_record('portfolio_survey', 'id', $this->survey->id)) {
            $name = $record->name;
            $email = $record->email;
        } else {
            $name = '';
            $email = '';
        }
        
        if(empty($email)) {
            return(false);
        }
    
        $answers = $this->generate_csv($rid);    
        $end = "\r\n<br>";   
        $subject = get_string('surveyresponse', 'portfolio') .": $name [$rid]";
        $body = get_string('surveyresponse', 'portfolio') .' "'.$name.'"'.$end;
        
        reset($answers);
            for ($i = 0; $i < count($answers[0]); $i++) {
                $sep = ' : ';
                switch($i) {
                case 1: 
                    $sep = ' ';
                    break;
                case 4:
                        $body .= get_string('user').' ';
                    break;
                case 6: 
                    if ($this->respondenttype != 'anonymous') {
                       $body .= get_string('email').$sep.$USER->email. $end;
                    }
                }
                $body .= $answers[0][$i].$sep.$answers[1][$i]. $end;
            }

        $mail = new PHPMailer();   
        $mail->Version = "Moodle $CFG->version";           // mailer version
        $mail->PluginDir = "$CFG->libdir/phpmailer/";      // plugin directory (eg smtp plugin)
    
        if (current_language() != "en") {
            $mail->CharSet = get_string("thischarset");
        }
    
        if ($CFG->smtphosts == "qmail") {
            $mail->IsQmail();                              // use Qmail system
    
        } else if (empty($CFG->smtphosts)) {
            $mail->IsMail();                               // use PHP mail() = sendmail
    
        } else {
            $mail->IsSMTP();                               // use SMTP directly
            if ($CFG->debug > 7) {
                echo "<pre>\n";
                $mail->SMTPDebug = true;
            }
            $mail->Host = "$CFG->smtphosts";               // specify main and backup servers
    
            if ($CFG->smtpuser) {                          // Use SMTP authentication
                $mail->SMTPAuth = true;
                $mail->Username = $CFG->smtpuser;
                $mail->Password = $CFG->smtppass;
            }
        }
    
        $mail->Sender   = $CFG->noreplyaddress;
        $mail->From     = $CFG->noreplyaddress;
        $mail->FromName = $name;
        $mail->Subject  =  $subject;
        $mail->AddAddress($email);
        $mail->WordWrap = 79;                               // set word wrap
    
        $mail->IsHTML(true);
        $mail->Encoding = "quoted-printable";           // Encoding to use
        $mail->Body    =  $body;
        $mail->AltBody =  "\n$body\n";
    
        if ($mail->Send()) {
            return true;
        } else {
            return false;
        }
    }
    
    function response_insert($sid, $section, $rid, $userid, &$formdata) {
        global $CFG;
    
        if(empty($rid)) {
            // create a uniqe id for this response
            $record = new object;
            $record->survey_id = $sid;
            $record->username = $userid;
            $rid = insert_record('portfolio_response', $record);
        }
    
        if (!empty($this->questionsbysec[$section])) {
            foreach ($this->questionsbysec[$section] as $question) {
                $question->insert_response($rid, $formdata);
            }
        }
        return($rid);
    }
    
    function response_select($rid, $col = null, $qids = null) {
        global $CFG;

        $sid = $this->survey->id;    
        $values = array();
    
        if ($col == null) {
            $col = '';
        }
        if (!is_array($col) && !empty($col)) {
            $col = explode(',', preg_replace("/\s/",'', $col));
        }
        if (is_array($col) && count($col) > 0) {
            $col = ',' . implode(',', array_map(create_function('$a','return "q.$a";'), $col));
        }
    
        if ($qids == null) {
            $qids = '';
        } elseif (is_array($qids)) {
            $qids = 'AND a.question_id ' . $this->array_to_insql($qids);
        } elseif (intval($qids) > 0) {
            $qids = 'AND a.question_id = ' . intval($qids);
        } else {
            $qids = '';
        }
    
        // --------------------- response_bool ---------------------
        $sql = 'SELECT q.id '.$col.', a.choice_id '.
               'FROM '.$CFG->prefix.'portfolio_response_bool a, '.$CFG->prefix.'portfolio_question q '.
               'WHERE a.response_id=\''.$rid.'\' AND a.question_id=q.id '.$qids;
        if ($records = get_records_sql($sql)) {
            foreach ($records as $qid => $row) {
                $choice = $row->choice_id;
                unset ($row->id);
                unset ($row->choice_id);
                $row = (array)$row;
                $newrow = array();
                foreach ($row as $key => $val) {
                    if (!is_numeric($key)) {
                        $newrow[] = $val;
                    }
                }
                $values[$qid] = $newrow;
                array_push($values["$qid"], ($choice == 'Y') ? get_string('yes') : get_string('no'), $choice);
            }
        }
    
        // --------------------- response_single ---------------------
        $sql = 'SELECT q.id '.$col.',c.content as ccontent,c.id as cid '.
               'FROM '.$CFG->prefix.'portfolio_response_single a, '.
                       $CFG->prefix.'portfolio_question q, '.
                       $CFG->prefix.'portfolio_question_choice c '.
               'WHERE a.response_id=\''.$rid.'\' AND a.question_id=q.id AND a.choice_id=c.id '.$qids;
        if ($records = get_records_sql($sql)) {
            foreach ($records as $qid => $row) {
                $cid = $row->cid;
                unset($row->id);
                unset($row->cid);
                $arow = get_object_vars($row);
                $newrow = array();
                foreach ($arow as $key => $val) {
                    if (!is_numeric($key)) {
                        $newrow[] = $val;
                    }
                }
                if (ereg('^!other', $row->ccontent)) {
                    $newrow[] = 'other_' . $cid;
                } else {
                    $newrow[] = (int)$cid;
                }
                $values[$qid] = $newrow;
            }
        }
    
        // --------------------- response_multiple ---------------------
        $sql = 'SELECT a.id as aid, q.id as qid '.$col.',c.content as ccontent,c.id as cid '.
               'FROM '.$CFG->prefix.'portfolio_response_multiple a, '.
                       $CFG->prefix.'portfolio_question q, '.$CFG->prefix.'portfolio_question_choice c '.
               'WHERE a.response_id=\''.$rid.'\' AND a.question_id=q.id AND a.choice_id=c.id '.$qids.' '.
               'ORDER BY a.id,a.question_id,c.id';
        $arr = array();
        $tmp = null;
        if ($records = get_records_sql($sql)) {
            foreach ($records as $aid => $row) {
                $qid = $row->qid;
                $cid = $row->cid;
                unset($row->aid);
                unset($row->qid);
                unset($row->cid);
                $arow = get_object_vars($row);
                $newrow = array();
                foreach ($arow as $key => $val) {
                    if (!is_numeric($key)) {
                        $newrow[] = $val;
                    }
                }
                if (ereg('^!other', $row->ccontent)) {
                    $newrow[] = 'other_' . $cid;
                } else {
                    $newrow[] = (int)$cid;
                }
                if($tmp == $qid) {
                    $arr[] = $newrow;
                    continue;
                }
                if($tmp != null) {
                    $values["$tmp"]=$arr;
                }
                $tmp = $qid;
                $arr = array($newrow);
            }
        }
        if($tmp != null) {
            $values["$tmp"]=$arr;
        }
        unset($arr);
        unset($tmp);
        unset($row);
        
            // --------------------- response_other ---------------------
            // this will work even for multiple !other fields within one question AND for identical !other responses in different questions JR
        $sql = 'SELECT c.id as cid, a.response as aresponse, q.id as qid '.
               'FROM '.$CFG->prefix.'portfolio_response_other a, '.$CFG->prefix.'portfolio_question q, '.
                       $CFG->prefix.'portfolio_question_choice c '.
               'WHERE a.response_id=\''.$rid.'\' AND a.question_id=q.id AND a.choice_id=c.id '.$qids.' '.
               'ORDER BY a.question_id,c.id';
        if ($records = get_records_sql($sql)) {
            foreach ($records as $record) {
                $cid = $record->cid;
                $qid = $record->qid;
                // the first two empty values are needed for compatibility with "normal" (non !other) responses
                // they are only needed for the CSV export, in fact - JR
                $newrow = array('','',$record->aresponse);
                $values["${qid}_${cid}"] = $newrow;
            }
        }    
        
            // --------------------- response_rank ---------------------
        $sql = 'SELECT a.id as aid, q.id AS qid, c.id AS cid '.$col.',c.content as ccontent,a.rank as arank '.
               'FROM '.$CFG->prefix.'portfolio_response_rank a, '.
                       $CFG->prefix.'portfolio_question q, '.
                       $CFG->prefix.'portfolio_question_choice c '.
               'WHERE a.response_id=\''.$rid.'\' AND a.question_id=q.id AND a.choice_id=c.id '.$qids.' '.
               'ORDER BY aid, a.question_id,c.id';
        if ($records = get_records_sql($sql)) {
            foreach ($records as $aid => $row) {
                /// Next two are 'qid' and 'cid', each with numeric and hash keys.
                $qid = $row->qid.'_'.$row->cid;
                unset($row->aid); // get rid of the answer id.
                unset($row->qid);
                unset($row->cid);
                $row = (array)$row;
                $newrow = array();
                foreach ($row as $key => $val) {
                    if (!is_numeric($key)) {
                        $newrow[] = $val;
                    }
                }
                settype($newrow[count($newrow) - 1], 'integer');
                $values[$qid] = $newrow;
            }
        }
    
            // --------------------- response_text ---------------------
        $sql = 'SELECT q.id '.$col.',a.response as aresponse '.
               'FROM '.$CFG->prefix.'portfolio_response_text a, '.$CFG->prefix.'portfolio_question q '.
               'WHERE a.response_id=\''.$rid.'\' AND a.question_id=q.id '.$qids;
        if ($records = get_records_sql($sql)) {
            foreach ($records as $qid => $row) {
                unset($row->id);
                $row = (array)$row;
                $newrow = array();
                foreach ($row as $key => $val) {
                    if (!is_numeric($key)) {
                        $newrow[] = $val;
                    }
                }
                $values["$qid"]=$newrow;
                $val = array_pop($values["$qid"]);
                array_push($values["$qid"], $val, $val);
            }
        }
    
            // --------------------- response_date ---------------------
        $sql = 'SELECT q.id '.$col.',a.response as aresponse '.
               'FROM '.$CFG->prefix.'portfolio_response_date a, '.$CFG->prefix.'portfolio_question q '.
               'WHERE a.response_id=\''.$rid.'\' AND a.question_id=q.id '.$qids;
        if ($records = get_records_sql($sql)) {
            $dateformat = get_string('strfdate', 'portfolio');
            foreach ($records as $qid => $row) {
                unset ($row->id);
                $row = (array)$row;
                $newrow = array();
                foreach ($row as $key => $val) {
                    if (!is_numeric($key)) {
                        $newrow[] = $val;
                    // convert date from yyyy-mm-dd database format to actual portfolio dateformat
                    // does not work with dates prior to 1900 under Windows
                        if (preg_match('/\d\d\d\d-\d\d-\d\d/', $val)) {
                            $dateparts = split('-', $val);
                            $val = mktime(0, 0, 0, $dateparts[1], $dateparts[2], $dateparts[0]); // Unix timestamp
                            $val = userdate ( $val, $dateformat);
                            $newrow[] = $val;
                        }
                    }
                }
                $values["$qid"]=$newrow;
                $val = array_pop($values["$qid"]);
                array_push($values["$qid"], '', '', $val);
            }
        }
    
            // --------------------- return ---------------------
//            uksort($values, 'portfolio_response_key_cmp');
            return($values);
    }
    
    function response_goto_thankyou() {
        global $CFG;

        $select = 'id = '.$this->survey->id;
        $fields = 'thanks_page,thank_head,thank_body';
        if ($result = get_record_select('portfolio_survey', $select, $fields)) {
            $thank_url = $result->thanks_page;
            $thank_head = $result->thank_head;
            $thank_body = $result->thank_body;
        } else {
            $thank_url = '';
            $thank_head = '';
            $thank_body = '';
        }
        if(!empty($thank_url)) {
            if(!headers_sent()) {
                header("Location: $thank_url");
                exit;
            }
    ?>
    <script language="JavaScript" type="text/javascript">
    <!--
    window.location="<?php echo($thank_url); ?>"
    //-->
    </script>
    <noscript>
    <h2 class="thankhead">Thank You for completing this survey.</h2>
    <blockquote class="thankbody">Please click
    <a class="returnlink" href="<?php echo($thank_url); ?>">here</a>
    to continue.</blockquote>
    </noscript>
    <?php
            exit;
        }
    
        if(empty($thank_body) && empty($thank_head)) {
            $thank_body = get_string('thank_body', 'portfolio', get_string('continue') );
            $thank_head = get_string('thank_head', 'portfolio');
        }
    ?>
    <h2 class="thankhead"><?php p($thank_head); ?></h2>
    <div class="thankbody"><?php echo(format_text($thank_body, FORMAT_HTML)); ?></div>
    <div class="returnLink"><a href="<?php echo($CFG->wwwroot.'/course/view.php?id='.$this->course->id); ?>">
        <?php echo(get_string('continue', 'moodle')); ?></a></div>
    <?php
        return;
    }
    
    function response_goto_saved($url) {
    ?>
    <div class="thankbody">
    <?php print_string('savedprogress', 'portfolio', '<strong>'.get_string('resumesurvey', 'portfolio').'</strong>'); ?>
        <div class="returnLink"><a href="<?php echo $url; ?>"><?php print_string('resumesurvey', 'portfolio'); ?></a>
        </div>
    </div>

    <?php
        global $CFG;
        echo ('<div class="returnLink"><a href="'.$CFG->wwwroot.'/course/view.php?id='.$this->course->id.'">'
        .get_string("continue", "moodle").'</a></div>');
    ?>
    <?php
        return;
    }


    /// Survey Results Methods

    function survey_results_navbar($curr_rid, $userid=false) {
        global $CFG;

        $select = 'survey_id='.$this->survey->id.' AND complete = \'N\'';
        if ($userid !== false) {
            $select .= ' AND username = \''.$userid.'\'';
        }
        if (!($responses = get_records_select('portfolio_response', $select, 'id', 'id,survey_id,submitted,username'))) {
            return;
        }
        $total = count($responses);
    
        $rids = array();
        $ridssub = array();
        $ridsusername = array();
        $i = 0;
        $curr_pos = -1;
        foreach ($responses as $response) {
            array_push($rids, $response->id);
            array_push($ridssub, $response->submitted);
            array_push($ridsusername, $response->username);
            if ($response->id == $curr_rid) {
                $curr_pos = $i;
            }
            $i++;
        }
    
        $prev_rid = ($curr_pos > 0) ? $rids[$curr_pos - 1] : null;
        $next_rid = ($curr_pos < $total - 1) ? $rids[$curr_pos + 1] : null;
        $rows_per_page = 1;
        $pages = ceil($total / $rows_per_page);
    
        $url = $CFG->wwwroot.'/mod/portfolio/report.php?where=results&amp;sid='.$this->survey->id;
    
        $mlink = create_function('$i,$r', 'return "<a href=\"'.$url.'&amp;rid=$r\">$i</a>";');
    
        $linkarr = array();
    
        $display_pos = 1;
        if ($prev_rid != null) {
            array_push($linkarr, "<a href=\"$url&amp;rid=$prev_rid\">".get_string('previous').'</a>');
        }
        $ruser = '';
        for ($i = 0; $i < $curr_pos; $i++) {
            if ($user = get_record('user', 'id', $ridsusername[$i])) {
                    $ruser = fullname($user);
            }
            $title = userdate($ridssub[$i]).' | ' .$ruser;
            array_push($linkarr, '<a href="'.$url.'&amp;rid='.$rids[$i].'" title="'.$title.'">'.$display_pos.'</a>');
            $display_pos++;
        }
        array_push($linkarr, '<b>'.$display_pos.'</b>');
        for (++$i; $i < $total; $i++) {
            if ($user = get_record('user', 'id', $ridsusername[$i])) {
                    $ruser = fullname($user);
            }
            $title = userdate($ridssub[$i]).' | ' .$ruser;
            $display_pos++;
            array_push($linkarr, '<a href="'.$url.'&amp;rid='.$rids[$i].'" title="'.$title.'">'.$display_pos.'</a>');

        }
        if ($next_rid != null) {
            array_push($linkarr, "<a href=\"$url&amp;rid=$next_rid\">".get_string('next').'</a>');
        }
        echo implode(' | ', $linkarr);
    }

    function survey_results_navbar_student($curr_rid, $userid, $instance, $resps) {
        $total = count($resps);
        $rids = array();
        $ridssub = array();
        $i = 0;
        $curr_pos = -1;
        foreach ($resps as $response) {
            array_push($rids, $response->id);
            array_push($ridssub, $response->submitted);
            if ($response->id == $curr_rid) {
                $curr_pos = $i;
            }
            $i++;
        }
        $prev_rid = ($curr_pos > 0) ? $rids[$curr_pos - 1] : null;
        $next_rid = ($curr_pos < $total - 1) ? $rids[$curr_pos + 1] : null;
        $rows_per_page = 1;
        $pages = ceil($total / $rows_per_page);
        $url = 'myreport.php?instance='.$instance.'&amp;user='.$userid.'&amp;action=vresp';  
        $linkarr = array();
        $display_pos = 1;
        if ($prev_rid != null) {
            $title = userdate($ridssub[$curr_pos - 1]);
            array_push($linkarr, '<a href="'.$url.'&amp;rid='.$prev_rid.'" title="'.$title.'">'.get_string('previous').'</a>');
        }
        for ($i = 0; $i < $curr_pos; $i++) {
            $title = userdate($ridssub[$i]);
            array_push($linkarr, '<a href="'.$url.'&amp;rid='.$rids[$i].'" title="'.$title.'">'.$display_pos.'</a>');
            $display_pos++;
        }
        array_push($linkarr, '<b>'.$display_pos.'</b>');
        for (++$i; $i < $total; $i++) {
            $display_pos++;
            $title = userdate($ridssub[$i]);
            array_push($linkarr, '<a href="'.$url.'&amp;rid='.$rids[$i].'" title="'.$title.'">'.$display_pos.'</a>');
        }
        if ($next_rid != null) {
//            $title = userdate($ridssub[$curr_pos]);
            $title = userdate($ridssub[$curr_pos + 1]);
            array_push($linkarr, '<a href="'.$url.'&amp;rid='.$next_rid.'" title="'.$title.'">'.get_string('next').'</a>');
        }
        echo implode(' | ', $linkarr);
    }

    /* {{{ proto string survey_results(int survey_id, int precision, bool show_totals, int question_id, array choice_ids, int response_id)
        Builds HTML for the results for the survey. If a
        question id and choice id(s) are given, then the results
        are only calculated for respodants who chose from the
        choice ids for the given question id.
        Returns empty string on sucess, else returns an error
        string. */
    function survey_results($precision = 1, $showTotals = 1, $qid = '', $cids = '', $rid = '', $guicross='', $uid=false) {
        global $CFG;

        $bg = '';
        if(empty($precision)) {
            $precision  = 1;
        }
        if($showTotals === '') {
            $showTotals = 1;
        }
            
        if(is_int($cids)) {
            $cids = array($cids);
        }
        if(is_string($cids)) {
            $cids = split(" ",$cids); // turn space seperated list into array
        }
    
        // set up things differently for cross analysis
        $cross = !empty($qid);
        if($cross) {
            if(is_array($cids) && count($cids)>0) {
                $cidstr = $this->array_to_insql($cids);
            } else {
                $cidstr = '';
            }
        }
    
        // build associative array holding whether each question
        // type has answer choices or not and the table the answers are in
        /// TO DO - FIX BELOW TO USE STANDARD FUNCTIONS
        $has_choices = array();
        $response_table = array();
        if (!($types = get_records('portfolio_question_type', '', '', 'id', 'id,has_choices,response_table'))) {
            $errmsg = sprintf('%s [ %s: question_type ]',
                    get_string('errortable', 'portfolio'), 'Table');
            return($errmsg);
        }
        foreach ($types as $type) {
            $has_choices[$type->id]=$type->has_choices;
            $response_table[$type->id]=$type->response_table;
        }

        // load survey title (and other globals)
        if (empty($this->survey)) {
            $errmsg = get_string('erroropening', 'portfolio') ." [ ID:${sid} R:";
            return($errmsg);
        }
    
        if (empty($this->questions)) {
            $errmsg = get_string('erroropening', 'portfolio') .' '. 'No questions found.' ." [ ID:${sid} ]";
            return($errmsg);
        }
    
        // find out more about the question we are cross analyzing on (if any)
        if($cross) {
            $crossTable = $response_table[get_field('portfolio_question', 'type_id', 'id', $qid)];
            if(!in_array($crossTable, array('response_single','response_bool','response_multiple'))) {
                $errmsg = get_string('errorcross', 'portfolio') .' [ '. 'Table' .": ${crossTable} ]";
                return($errmsg);
            }
        }
    
    // find total number of survey responses
    // and relevant response ID's
        if (!empty($rid)) {
            $rids = $rid;
            if (is_array($rids)) {
                $ridstr = "IN (" . ereg_replace("([^,]+)","'\\1'", join(",", $rids)) .")";
                $navbar = false;
            } else {
                $ridstr = "= '${rid}'";
                $navbar = true;
            }
            $total = 1;
        } else {
            $navbar = false;
            $sql = "";
            if($cross) {
                if(!empty($cidstr))
                    $sql = "SELECT A.response_id, R.id
                              FROM ".$CFG->prefix.'portfolio_'.$crossTable." A,
                                   ".$CFG->prefix."portfolio_response R
                             WHERE A.response_id=R.id AND
                                   R.complete='Y' AND
                                   A.question_id='${qid}' AND
                                   A.choice_id ${cidstr}
                             ORDER BY A.response_id";
                else
                    $sql = "SELECT A.response_id, R.id
                              FROM ".$CFG->prefix.'portfolio_'.$crossTable." A,
                                   ".$CFG->prefix."portfolio_response R
                             WHERE A.response_id=R.id AND
                                   R.complete='Y' AND
                                   A.question_id='${qid}' AND
                                   A.choice_id = 0
                             ORDER BY A.response_id";
            } else if ($uid !== false) {
                $sql = "SELECT r.id, r.survey_id
                          FROM ".$CFG->prefix."portfolio_response r
                         WHERE r.survey_id='{$this->survey->id}' AND
                               r.username = $uid AND
                               r.complete='N'
                         ORDER BY r.id";
            } else {
                $sql = "SELECT R.id, R.survey_id
                          FROM ".$CFG->prefix."portfolio_response R
                         WHERE R.survey_id='{$this->survey->id}' AND
                               R.complete='N'
                         ORDER BY R.id";
            }
            if (!($rows = get_records_sql($sql))) {
                $errmsg = get_string('erroropening', 'portfolio') ." [ ID:${sid} ] [ ";
                return($errmsg);
            }
            $total = count($rows);
            if($total < 1) {
                $errmsg = get_string('erroropening', 'portfolio') .' '. get_string('noresponsedata', 'portfolio') ." [ ID:${sid} ]";
                    return($errmsg);
            }
    
            $rids = array();
            foreach ($rows as $row) {
                array_push($rids, $row->id);
            }
            // create a string suitable for inclusion in a SQL statement
            // containing the desired Response IDs
            // ex: "('304','311','317','345','365','370','403','439')"
            $ridstr = "IN (" . ereg_replace("([^,]+)","'\\1'", join(",", $rids)) .")";
        }
        
        if ($navbar) {
            // show response navigation bar
            $this->survey_results_navbar($rid);
        }
    
    ?>
    <h2><?php echo($this->survey->title); ?></h2>
    <h3><?php echo($this->survey->subtitle); ?></h3>
    <?php echo($this->survey->info); ?>
    <?php
        if($cross) {
            echo("<blockquote>" ._('Cross analysis on QID:') ." ${qid}</blockquote>\n");
        }
    ?>
    <table border="0" style="width:100%">
    <?php
        $i=0; // question number counter
        foreach ($this->questions as $question) {
            // process each question
            $totals = $showTotals;
    
            if ($question->type_id == 99) {
                echo("<tr><td></td></tr>\n");
                continue;
            }
            if ($question->type_id == 100) {
                echo("<tr><td>". $question->content ."</td></tr>\n");
                continue;
            }
    ?>
            <tr>
            <td>
    <?php
            if ($question->type_id < 50) {
                if (!empty($guicross)){
                    echo ('<div>');
                    echo ('<input type="hidden" name="where" value="results" />');
                    echo ('<input type="hidden" name="sid" value="'.$this->survey->id.'" />');
                    echo ('</div>');
                    echo ("\n<table width=\"90%\" border=\"0\">\n");
                    echo ("<tbody>\n");
                    echo ("   <tr>\n");
                    echo ("      <td width=\"34\" height=\"31\" >\n");
                    if ($question->type_id ==1 || $question->type_id ==4 || $question->type_id ==5 || $question->type_id ==6){
                        echo ("<div align=\"center\">\n");
                        echo ("   <input type=\"radio\" name=\"qid\" value=\"".$question->id."\" />\n");
                        echo ("</div>\n");
                    }
                    echo ("</td>\n");
                    echo ("<td width=\"429\" >\n");
                } //end if empty($guicross)
                echo ("<div class = \"reportQuestionTitle\"><b>".++$i.".</b>&nbsp;");
                if (!empty($guicross)){
                    echo ("</td>\n");
                    echo ("<td width=\"33\" >\n");
                    if ($question->type_id ==1 || $question->type_id ==4 || $question->type_id ==5 || $question->type_id ==6){
                        echo ("<div align=\"center\">\n");
                        echo ("<input type=\"radio\" name=\"qidr\" value=\"".$question->id."\" />\n");
                        echo ("</div>\n");
                    }
                    echo ("</td>\n");
                    echo ("<td width=\"32\" >\n");
                    if ($question->type_id ==1 || $question->type_id ==4 || $question->type_id ==5 || $question->type_id ==6){
                        echo ("<div align=\"center\">\n");
                        echo ("<input type=\"radio\" name=\"qidc\" value=\"".$question->id."\" />\n");
                        echo ("</div>\n");
                    }
                    echo ("</td>\n");
                    echo ("</tr>\n");
                    echo ("</tbody>\n");
                    echo ("</table>\n");
                } //end if empty($guicross)
            } //end if ($question->type_id  < 50)
    
    $counts = array();

    // ---------------------------------------------------------------------------
 
    $question->display_results($rids, $guicross);
 
    ?>
            </td>
        </tr>
    <?php } // end while ?>
    </table>
    <?php
        return;
    }

/* {{{ proto array survey_generate_csv(int survey_id)
    Exports the results of a survey to an array.
    */
    function generate_csv($rid='') {
	    global $CFG;
        $output = array();

        $columns = array(
                get_string('response','portfolio'),
                get_string('submitted','portfolio'),
                get_string('course'),
                get_string('group'),
                get_string('id','portfolio'),
                get_string('fullname'),
                get_string('username'),
            );
    
        $types = array(
                0,
                0,
                1,
                1,
                0,
                1,
                1,
            );
    
        $arr = array();
    
        $id_to_csv_map = array(
            '0',    // 0: unused
            '1',    // 1: bool -> boolean
            '1',    // 2: text -> string 
            '1',    // 3: essay -> string
            '1',    // 4: radio -> string
            '1',    // 5: check -> string
            '1',    // 6: dropdn -> string
            '0',    // 7: rating -> number
            '0',    // 8: rate -> number
            '1',    // 9: date -> string
            '0'     // 10: numeric -> number
        );
    
        if (!$survey = get_record('portfolio_survey', 'id', $this->survey->id)) {
            error (get_string('surveynotexists', 'portfolio'));
        }
    
        $select = 'survey_id = '.$this->survey->id.' AND deleted = \'N\' AND type_id < 50';
        $fields = 'id,name,type_id,position';
        if (!($records = get_records_select('portfolio_question', $select, 'position', $fields))) {
            $records = array();
        }
        global $CFG;
        $num = 1;
        foreach ($records as $record) {
            // establish the table's field names
            $qid = $record->id;
            if ($record->name == '') {
                $record->name = '_qu_'.sprintf("%02d", $record->position);              
            }
            $col = $record->name;
            $type = $record->type_id;
            if ($type == 8) {
                /* rate */
                $sql = "SELECT c.id as cid, q.id, q.name, c.content 
                FROM ".$CFG->prefix."portfolio_question q ".
                'LEFT JOIN '.$CFG->prefix."portfolio_question_choice c ON question_id = q.id ".
                'WHERE q.id = '.$qid;
                if (!($records2 = get_records_sql($sql))) {
                    $records2 = array();
                }
                $subqnum = 0;
                foreach ($records2 as $record2) {
                    $nameddegrees = 0;
                    $content = $record2->content;
                    if (strpos($content,'=',1) && is_numeric(substr($content,0,1))) {
                        $nameddegrees++;
                    } else {
                        if ($record2->name == '') {
                            $subqnum ++;
                            $record2->name = '_qu_'.sprintf("%02d",$record->position);
                            $record2->content = $subqnum;
                        }
                        $col = $record2->name.'->'.$record2->content;
                        array_push($columns, $col);
                        array_push($types, $id_to_csv_map[$type]);
                    }
                }
            } else {
                if (in_array($col, $columns)) {  
                 $col = $col.$num;
                }
                array_push($columns, $col);
                array_push($types, $id_to_csv_map[$type]);
            }
            $num++; 
        }
        array_push($output, $columns);
        $numcols = count($output[0]);
        // send e-mail for a unique response ($rid)
        $andrid = '';
        if ($rid) {
            $andrid = " AND id = $rid";
        }
        $select = 'survey_id = '.$this->survey->id.' AND complete=\'N\''.$andrid;
        $fields = 'id,submitted,username';
        if (!($records = get_records_select('portfolio_response', $select, 'submitted', $fields))) {
            $records = array();
        }
        $isanonymous = $this->respondenttype == 'anonymous';
        foreach ($records as $record) {
            // get the response
            $response = $this->response_select_name($record->id);
            $qid = $record->id;
            //JR for better compabitility & readability with Excel
            $submitted = date(get_string('strfdateformatcsv', 'portfolio'), $record->submitted); 
            if ($isanonymous) {
                $username = get_string('anonymous', 'portfolio');        
            } else {
                $username  = $record->username;
            }

            /// Moodle:
            //  Get the course name that this portfolio belongs to.
            if ($survey->realm != 'public') {
                $courseid = $this->course->id;
                $coursename = $this->course->fullname;
            } else {
                /// For a public portfolio, look for the course that used it.
                $sql = 'SELECT q.id, q.course, c.fullname '.
                       'FROM '.$CFG->prefix.'portfolio q, '.$CFG->prefix.'portfolio_attempts qa, '.
                            $CFG->prefix.'course c '.
                       'WHERE qa.rid = '.$qid.' AND q.id = qa.qid AND c.id = q.course';
                if ($record = get_record_sql($sql)) {
                    $courseid = $record->course;
                    $coursename = $record->fullname;
                } else {
                    $courseid = $this->course->id;
                    $coursename = $this->course->fullname;
                }
            }

            /// Moodle:
            //  If the username is numeric, try it as a Moodle user id.
            if (is_numeric($username)) {
                if ($user = get_record('user', 'id', $username)) {
                    $uid = $username;
                    $fullname = fullname($user);
                    $username = $user->username;
                } else {
                    $uid = '';
                    $fullname =  get_string('anonymous', 'portfolio');
                    $username = '';
                }
            } else {
                $uid = '';
                    $fullname =  get_string('anonymous', 'portfolio');
                    $username = '';
            }
    
            /// Moodle:
            //  Determine if the user is a member of a group in this course or not.
            $groupname = "";
            if (!$isanonymous) {
                if ($groups = get_groups($courseid, $uid)) {
                    $group = current($groups);
                    $groupname = $group->name;
                } else {
                    $groupname = get_string('no') . ' ' . get_string('group');
                }
            }
            $arr = array();
            array_push($arr, $qid);
            array_push($arr, $submitted);
            array_push($arr, $coursename);
            array_push($arr, $groupname);
            array_push($arr, $uid);
            array_push($arr, $fullname);
            array_push($arr, $username);
    
            // merge it

            for($i = 7; $i < $numcols; $i++) {
                if (isset($response[$columns[$i]]) && is_array($response[$columns[$i]])) {
                    $response[$columns[$i]] = join(',', $response[$columns[$i]]);                    
                }
                switch ($types[$i]) {
                case 1: /* string */
                    if (isset($response[$columns[$i]])) {
                        /* Excel seems to allow "\n" inside a quoted string, but
                         * "\r\n" is used as a record separator and so "\r" may
                         * not occur within a cell. So if one would like to preserve
                         * new-lines in a response, remove the "\n" from the
                         * regex below.
                         */
                        // email format text is plain text for being displayed in Excel, etc. added by JR
                        // but it must be stripped of carriage returns
                        $response[$columns[$i]] = format_text_email($response[$columns[$i]], FORMAT_HTML);
                        $response[$columns[$i]] = ereg_replace("[\r\n\t]", ' ', $response[$columns[$i]]);
                        $response[$columns[$i]] = ereg_replace('"', '""', $response[$columns[$i]]);
                        $response[$columns[$i]] = '"'. $response[$columns[$i]] .'"';
                    }
                    /* fall through */
                case 0: /* number */
                    if (isset($response[$columns[$i]]))
                        array_push($arr, $response[$columns[$i]]);
                    else
                        array_push($arr, '');
                    break;
                }
            }
            array_push($output, $arr);
        }

        // change table headers to incorporate actual question numbers
        $numcol = 0;
        $numquestion = 0;
        $thisname = '';
        $out = '';
        foreach($output[0] as $thisoutput) {
            // skip first 7 columns (response id, submitted date, etc.)
            if ($numcol < 7) {
                $numcol++;
                continue;
            }
            $sep = '_';
            $numquestion++;
            if($pos = strpos($thisoutput, '->')) {
                if ($thisname != substr($thisoutput,0,$pos)) {
                    $thisname = substr($thisoutput,0,$pos);
                } else {
                    $numquestion--;
                }
            }
            if (substr($thisoutput,0,4) == '_qu_') {
                $out = 'Q'.sprintf("%02d", $numquestion).$sep.substr($thisoutput,8);
            } else {
                $out = 'Q'.sprintf("%02d", $numquestion).$sep.$thisoutput;                
            }
            $output[0][$numcol] = rtrim($out, '_');                           
            $numcol++;
        }
        return $output;
    }

    /* {{{ proto bool survey_export_csv(int survey_id, string filename)
        Exports the results of a survey to a CSV file.
        Returns true on success.
        */
    function export_csv($filename) {
        $umask = umask(0077);
        $fh = fopen($filename, 'w');
        umask($umask);
        if(!$fh)
            return 0;
    
        $data = survey_generate_csv();
    
        foreach ($data as $row) {
            fputs($fh, join(',', $row) . "\n");
        }
    
        fflush($fh);
        fclose($fh);
    
        return 1;
    }
}
    /* {{{ proto void mkcrossformat (array weights, integer qid)
       Builds HTML to allow for cross tabulation/analysis reporting.
     */
function portfolio_response_key_cmp($l, $r) {
    $lx = explode('_', $l);
    $rx = explode('_', $r);
    $lc = intval($lx[0]);
    $rc = intval($rx[0]);
    if ($lc == $rc) {
        if (count($lx) > 1 && count($rx) > 1) {
            $lc = intval($lx[1]);
            $rc = intval($rx[1]);
        } else if (count($lx) > 1) {
            $lc++;
        } else if (count($rx) > 1) {
            $rc++;
        }
    }
    if ($lc == $rc)
        return 0;
    return ($lc > $rc) ? 1 : -1;
}

    function portfolio_check_date ($thisdate) {
        $dateformat = get_string('strfdate', 'portfolio'); 
        if (preg_match('/(%[mdyY])(.+)(%[mdyY])(.+)(%[mdyY])/', $dateformat, $matches)) {
            $date_pieces = explode($matches[2], $thisdate);
            if (count($date_pieces) > 1) {
                if ($matches[1] == '%m') $month = $date_pieces[0];
                if ($matches[1] == '%d') $day = $date_pieces[0];
                if ($matches[1] == '%y') $year = strftime('%C').$date_pieces[0];
                if ($matches[1] == '%Y') $year = $date_pieces[0];
    
                if ($matches[2] != $matches[4]) {
                    $date_pieces = explode($date_pieces[1], $matches[4]);
                    $date_pieces[2] = $date_pieces[1]; $date_pieces[1] = $date_pieces[0];
                }
    
                if ($matches[3] == '%m') $month = $date_pieces[1];
                if ($matches[3] == '%d') $day = $date_pieces[1];
                if ($matches[3] == '%y') $year = strftime('%C').$date_pieces[1];
                if ($matches[3] == '%Y') $year = $date_pieces[1];
    
                if ($matches[5] == '%m') $month = $date_pieces[2];
                if ($matches[5] == '%d') $day = $date_pieces[2];
                if ($matches[5] == '%y') $year = strftime('%C').$date_pieces[2];
                if ($matches[5] == '%Y') $year = $date_pieces[2];
    
                $month = min(12,$month);
                $month = max(1,$month);
                if ($month == 2) {
                    $day = min(29, $day);
                } else if ($month == 4 || $month == 6 || $month == 9 || $month == 11) {    
                    $day = min(30, $day);
                } else {
                    $day = min(31, $day);
                }
                $day = max(1, $day);
                if (!$thisdate = mktime(0, 0, 0, $month, $day, $year)) {
                    notify(get_string('wrongdaterange', 'portfolio'));
                    $thisdate = '';
                } else {
                    $thisdate = trim(userdate ($thisdate, $dateformat));
                }
                return $thisdate;
            } else {
//                $today = userdate ( time(), portfolioDATEFORMAT);
                return;
            }
        }
    }
    // a variant of Moodle's notify function, with a different formatting
    function portfolio_notify($message) {
        $message = clean_text($message);
        $output = '<div class="errorbox errorboxcontent">'. $message .'</div>';
        echo $output;
    }    
?>